@@ -120,6 +120,41 @@ void ai_data_analyze_rulesets(struct player *pplayer)
   ai_data_city_impr_calc(pplayer, ai);
 }
 
+/**************************************************************************
+  This function is called each turn to initialize pplayer->ai.stats.units.
+**************************************************************************/
+static void count_my_units(struct player *pplayer)
+{
+  struct ai_data *ai = ai_data_get(pplayer);
+
+  memset(&ai->stats.units, 0, sizeof(ai->stats.units));
+
+  unit_list_iterate(pplayer->units, punit) {
+    switch (unit_type(punit)->move_type) {
+    case LAND_MOVING:
+      ai->stats.units.land++;
+      break;
+    case SEA_MOVING:
+      ai->stats.units.sea++;
+      break;
+    case HELI_MOVING:
+    case AIR_MOVING:
+      ai->stats.units.air++;
+      break;
+    }
+
+    if (unit_flag(punit, F_TRIREME)) {
+      ai->stats.units.triremes++;
+    }
+    if (unit_flag(punit, F_MISSILE)) {
+      ai->stats.units.missiles++;
+    }
+    if (can_upgrade_unittype(pplayer, punit->type) >= 0) {
+      ai->stats.units.upgradeable++;
+    }
+  } unit_list_iterate_end;
+}
+
 /**************************************************************************
   Make and cache lots of calculations needed for other functions.
 
@@ -402,6 +437,8 @@ void ai_data_turn_init(struct player *pplayer)
       *punit->ai.cur_pos = punit->tile;
     } unit_list_iterate_end;
   } players_iterate_end;
+
+  count_my_units(pplayer);
 }
 
 /**************************************************************************
