@@ -72,6 +72,7 @@
 #include "version.h"
 
 #include "agents.h"
+#include "attribute.h"
 #include "chatline_g.h"
 #include "civclient.h"
 #include "climisc.h"
@@ -122,7 +123,7 @@ static void close_socket_callback(struct connection *pc)
 {
   close_socket_nomessage(pc);
   /* If we lost connection to the internal server - kill him */
-  client_kill_server();
+  client_kill_server(TRUE);
   append_output_window(_("Lost connection to server!"));
   freelog(LOG_NORMAL, "lost connection to server");
 }
@@ -239,10 +240,18 @@ int try_to_connect(const char *username, char *errbuf, int errbufsize)
 **************************************************************************/
 void disconnect_from_server(void)
 {
-  close_socket_nomessage(&aconnection);
+  const bool force = !aconnection.used;
+
+  attribute_flush();
   /* If it's internal server - kill him 
    * We assume that we are always connected to the internal server  */
-  client_kill_server();
+  if (!force) {
+    client_kill_server(FALSE);
+  }
+  close_socket_nomessage(&aconnection);
+  if (force) {
+    client_kill_server(TRUE);
+  }
   append_output_window(_("Disconnected from server."));
 }  
 
@@ -813,13 +822,13 @@ int begin_lanserver_scan(void)
   Listens for UDP packets broadcasted from a server that responded
   to the request-packet sent from the client. 
 **************************************************************************/
-struct server_list *get_lan_server_list(void) {
-
-# if defined(__VMS) && !defined(_DECC_V4_SOURCE)
-    size_t fromlen;
-# else
-    int fromlen;
-# endif
+struct server_list *get_lan_server_list(void)
+{
+#ifdef HAVE_SOCKLEN_T
+  socklen_t fromlen;
+#else
+  int fromlen;
+#endif
   union my_sockaddr fromend;
   struct hostent *from;
   char msgbuf[128];
@@ -834,6 +843,8 @@ struct server_list *get_lan_server_list(void) {
   fd_set readfs, exceptfs;
   struct timeval tv;
 
+ again:
+
   FD_ZERO(&readfs);
   FD_ZERO(&exceptfs);
   FD_SET(socklan, &exceptfs);
@@ -864,7 +875,7 @@ struct server_list *get_lan_server_list(void) {
 
     dio_get_uint8(&din, &type);
     if (type != SERVER_LAN_VERSION) {
-      return lan_servers;
+      goto again;
     }
     dio_get_string(&din, servername, sizeof(servername));
     dio_get_string(&din, port, sizeof(port));
@@ -883,7 +894,7 @@ struct server_list *get_lan_server_list(void) {
     server_list_iterate(*lan_servers, aserver) {
       if (!mystrcasecmp(aserver->host, servername) 
           && !mystrcasecmp(aserver->port, port)) {
-        return lan_servers;
+	goto again;
       } 
     } server_list_iterate_end;
 
@@ -900,9 +911,9 @@ struct server_list *get_lan_server_list(void) {
     pserver->players = NULL;
 
     server_list_insert(lan_servers, pserver);
-  } else {
-    return lan_servers;
-  }                                       
+
+    goto again;
+  }
 
   return lan_servers;
 }
