@@ -20,7 +20,7 @@ Freeciv - Copyright (C) 2003 - The Freeciv Project
 #endif
 
 bool client_start_server(void);
-void client_kill_server(void);
+void client_kill_server(bool force);
 
 bool is_server_running(void);
 bool can_client_access_hack(void);
@@ -29,8 +29,6 @@ void send_client_wants_hack(const char *filename);
 void send_start_saved_game(void);
 void send_save_game(char *filename);
 
-void disconnected_from_local_server(void);
-
 extern char player_name[MAX_LEN_NAME];
 extern char *current_filename;
 
