@@ -455,6 +455,14 @@ void set_units_in_combat(struct unit *pattacker, struct unit *pdefender)
 {
   punit_attacking = pattacker;
   punit_defending = pdefender;
+
+  if (punit_attacking == punit_focus || punit_defending == punit_focus) {
+    /* If one of the units is the focus unit, make sure hidden-focus is
+     * disabled.  We don't just do this as a check later because then
+     * with a blinking unit it would just disappear again right after the
+     * battle. */
+    set_focus_unit_hidden_state(FALSE);
+  }
 }
 
 /**************************************************************************
@@ -1424,7 +1432,7 @@ void do_map_click(struct tile *ptile, enum quickselect_type qtype)
       do_unit_goto(ptile);
       break;
     case HOVER_NUKE:
-      if (3 * real_map_distance(punit->tile, ptile)
+      if (SINGLE_MOVE * real_map_distance(punit->tile, ptile)
 	  > punit->moves_left) {
         append_output_window(_("Game: Too far for this unit."));
       } else {
