@@ -256,7 +256,6 @@ static void bribe_response(GtkWidget *w, gint response)
 			    diplomat_target_id, 0);
   }
   gtk_widget_destroy(w);
-  gtk_widget_destroy(diplomat_dialog);
 }
 
 /****************************************************************
@@ -267,6 +266,7 @@ static void diplomat_bribe_callback(GtkWidget *w, gpointer data)
   if (find_unit_by_id(diplomat_id) && find_unit_by_id(diplomat_target_id)) {
     dsend_packet_unit_bribe_inq(&aconnection, diplomat_target_id);
   }
+  gtk_widget_destroy(diplomat_dialog);
 }
 
 /****************************************************************
@@ -336,7 +336,6 @@ static void spy_sabotage_unit_callback(GtkWidget *w, gpointer data)
 {
   request_diplomat_action(SPY_SABOTAGE_UNIT, diplomat_id,
 			  diplomat_target_id, 0);
-
   gtk_widget_destroy(diplomat_dialog);
 }
 
@@ -392,8 +391,6 @@ static void spy_advances_response(GtkWidget *w, gint response, gpointer data)
   }
   gtk_widget_destroy(spy_tech_shell);
   spy_tech_shell = NULL;
-
-  gtk_widget_destroy(diplomat_dialog);
 }
 
 /****************************************************************
@@ -424,7 +421,7 @@ static void create_advances_list(struct player *pplayer,
 				 struct player *pvictim)
 {  
   GtkWidget *sw, *label, *vbox, *view;
-  int i, j;
+  int i;
   GtkListStore *store;
   GtkCellRenderer *rend;
   GtkTreeViewColumn *col;
@@ -481,8 +478,6 @@ static void create_advances_list(struct player *pplayer,
   gtk_container_add(GTK_CONTAINER(vbox), sw);
 
   /* Now populate the list */
-  j = 0;
-
   if (pvictim) { /* you don't want to know what lag can do -- Syela */
     GtkTreeIter it;
     GValue value = { 0, };
@@ -499,19 +494,16 @@ static void create_advances_list(struct player *pplayer,
 	gtk_list_store_set_value(store, &it, 0, &value);
 	g_value_unset(&value);
 	gtk_list_store_set(store, &it, 1, i, -1);
-        j++;
       }
     }
 
-    if(j > 0) {
-      gtk_list_store_append(store, &it);
+    gtk_list_store_append(store, &it);
 
-      g_value_init(&value, G_TYPE_STRING);
-      g_value_set_static_string(&value, _("At Spy's Discretion"));
-      gtk_list_store_set_value(store, &it, 0, &value);
-      g_value_unset(&value);
-      gtk_list_store_set(store, &it, 1, game.num_tech_types, -1);
-    }
+    g_value_init(&value, G_TYPE_STRING);
+    g_value_set_static_string(&value, _("At Spy's Discretion"));
+    gtk_list_store_set_value(store, &it, 0, &value);
+    g_value_unset(&value);
+    gtk_list_store_set(store, &it, 1, game.num_tech_types, -1);
   }
 
   gtk_dialog_set_response_sensitive(GTK_DIALOG(spy_tech_shell),
@@ -543,8 +535,6 @@ static void spy_improvements_response(GtkWidget *w, gint response, gpointer data
   }
   gtk_widget_destroy(spy_sabotage_shell);
   spy_sabotage_shell = NULL;
-
-  gtk_widget_destroy(diplomat_dialog);
 }
 
 /****************************************************************
@@ -679,6 +669,7 @@ pvictim to NULL and account for !pvictim in create_advances_list. -- Syela */
     create_advances_list(game.player_ptr, pvictim);
     gtk_window_present(GTK_WINDOW(spy_tech_shell));
   }
+  gtk_widget_destroy(diplomat_dialog);
 }
 
 /****************************************************************
@@ -692,6 +683,7 @@ static void spy_request_sabotage_list(GtkWidget *w, gpointer data)
     request_diplomat_action(SPY_GET_SABOTAGE_LIST, diplomat_id,
 			    diplomat_target_id, 0);
   }
+  gtk_widget_destroy(diplomat_dialog);
 }
 
 /****************************************************************
@@ -714,6 +706,7 @@ static void diplomat_incite_callback(GtkWidget *w, gpointer data)
   if (find_unit_by_id(diplomat_id) && find_city_by_id(diplomat_target_id)) {
     dsend_packet_city_incite_inq(&aconnection, diplomat_target_id);
   }
+  gtk_widget_destroy(diplomat_dialog);
 }
 
 /****************************************************************
@@ -726,7 +719,6 @@ static void incite_response(GtkWidget *w, gint response)
 			    diplomat_target_id, 0);
   }
   gtk_widget_destroy(w);
-  gtk_widget_destroy(diplomat_dialog);
 }
 
 /****************************************************************
@@ -2123,7 +2115,7 @@ static void races_response(GtkWidget *w, gint response, gpointer data)
   } else {
     popdown_races_dialog();
     disconnect_from_server();
-    client_kill_server();
+    client_kill_server(FALSE);
   }
 }
 
