@@ -17,6 +17,11 @@
 
 #include "chatline_g.h"
 
+#define	MAX_CHATLINE_HISTORY 20
+
+extern struct genlist history_list;
+extern int history_pos;
+
 void inputline_return(GtkWidget *w, gpointer data);
 void set_output_window_text(const char *text);
 
