@@ -15,42 +15,41 @@
 #include <config.h>
 #endif
 
-#include <gdk/gdkkeysyms.h>
 #include <assert.h>
 
-#include "fcintl.h"
+#include <gdk/gdkkeysyms.h>
+
 #include "events.h"
+#include "fcintl.h"
 #include "game.h"
 #include "mem.h"
 #include "support.h"
 
 #include "chatline_g.h"
 #include "citydlg_g.h"
+#include "civclient.h"
+#include "cma_fec.h"
+#include "messagewin_g.h"
+
 #include "cityrep.h"
 #include "dialogs.h"
-#include "inputdlg.h"
 #include "gui_stuff.h"
-#include "messagewin_g.h"
-#include "cma_fec.h"
+#include "inputdlg.h"
 
 #include "cma_fe.h"
 
 #define BUFFER_SIZE             64
 
-#define SPECLIST_TAG cma_dialog
+#define SPECLIST_TAG dialog
 #define SPECLIST_TYPE struct cma_dialog
 #include "speclist.h"
 
-#define SPECLIST_TAG cma_dialog
-#define SPECLIST_TYPE struct cma_dialog
-#include "speclist_c.h"
+#define dialog_list_iterate(dialoglist, pdialog) \
+    TYPED_LIST_ITERATE(struct cma_dialog, dialoglist, pdialog)
+#define dialog_list_iterate_end  LIST_ITERATE_END
 
-#define cma_dialog_list_iterate(presetlist, ppreset) \
-    TYPED_LIST_ITERATE(struct cma_dialog, presetlist, ppreset)
-#define cma_dialog_list_iterate_end  LIST_ITERATE_END
-
-static struct cma_dialog_list cma_dialog_list;
-static int cma_dialog_list_has_been_initialised;
+static struct dialog_list dialog_list;
+static bool dialog_list_has_been_initialised = FALSE;
 
 static int allow_refreshes = 1;
 
@@ -76,17 +75,17 @@ static void cma_select_preset_callback(GtkWidget *w, gint row, gint col,
 				       GdkEvent *ev, gpointer data);
 
 static void hscale_changed(GtkAdjustment *get, gpointer data);
-static void set_hscales(const struct cma_parameter *const parameter,
+static void set_hscales(const struct cm_parameter *const parameter,
 			struct cma_dialog *pdialog);
 
 /**************************************************************************
 ...
 **************************************************************************/
-static void ensure_initialised_cma_dialog_list(void)
+static void ensure_initialised_dialog_list(void)
 {
-  if (!cma_dialog_list_has_been_initialised) {
-    cma_dialog_list_init(&cma_dialog_list);
-    cma_dialog_list_has_been_initialised = 1;
+  if (!dialog_list_has_been_initialised) {
+    dialog_list_init(&dialog_list);
+    dialog_list_has_been_initialised = TRUE;
   }
 }
 
@@ -97,7 +96,7 @@ void close_cma_dialog(struct city *pcity)
 {
   struct cma_dialog *pdialog = get_cma_dialog(pcity);
 
-  cma_dialog_list_unlink(&cma_dialog_list, pdialog);
+  dialog_list_unlink(&dialog_list, pdialog);
 
   if (pdialog->name_shell) {
     gtk_widget_destroy(pdialog->name_shell);
@@ -116,13 +115,13 @@ void close_cma_dialog(struct city *pcity)
 *****************************************************************/
 struct cma_dialog *get_cma_dialog(struct city *pcity)
 {
-  ensure_initialised_cma_dialog_list();
+  ensure_initialised_dialog_list();
 
-  cma_dialog_list_iterate(cma_dialog_list, pdialog) {
+  dialog_list_iterate(dialog_list, pdialog) {
     if (pdialog->pcity == pcity) {
       return pdialog;
     }
-  } cma_dialog_list_iterate_end;
+  } dialog_list_iterate_end;
 
   return NULL;
 }
@@ -133,10 +132,10 @@ struct cma_dialog *get_cma_dialog(struct city *pcity)
 struct cma_dialog *create_cma_dialog(struct city *pcity, GtkAccelGroup *accel)
 {
   struct cma_dialog *pdialog;
-  struct cma_parameter param;
+  struct cm_parameter param;
   GtkWidget *frame, *page, *hbox, *label, *table;
   GtkWidget *vbox, *scrolledwindow, *hscale;
-  static gchar *preset_title_[] = { N_("Presets") };
+  static const char *preset_title_[] = { N_("Presets") };
   static gchar **preset_title = NULL;
   int i;
 
@@ -225,7 +224,7 @@ struct cma_dialog *create_cma_dialog(struct city *pcity, GtkAccelGroup *accel)
   gtk_table_attach_defaults(GTK_TABLE(table), label, 2, 3, 0, 1);
 
   for (i = 0; i < NUM_STATS; i++) {
-    label = gtk_label_new(cma_get_stat_name(i));
+    label = gtk_label_new(cm_get_stat_name(i));
     gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, i + 1, i + 2);
     gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
 
@@ -242,7 +241,7 @@ struct cma_dialog *create_cma_dialog(struct city *pcity, GtkAccelGroup *accel)
 		       pdialog);
 
     pdialog->factor[i] =
-	GTK_ADJUSTMENT(gtk_adjustment_new(1, 1, 25, 1, 1, 0));
+	GTK_ADJUSTMENT(gtk_adjustment_new(1, 0, 25, 1, 1, 0));
 
     hscale = gtk_hscale_new(GTK_ADJUSTMENT(pdialog->factor[i]));
     gtk_table_attach_defaults(GTK_TABLE(table), hscale, 2, 3, i + 1, i + 2);
@@ -274,7 +273,7 @@ struct cma_dialog *create_cma_dialog(struct city *pcity, GtkAccelGroup *accel)
 		     GTK_SIGNAL_FUNC(hscale_changed), pdialog);
 
   pdialog->factor[NUM_STATS] =
-      GTK_ADJUSTMENT(gtk_adjustment_new(1, 1, 50, 1, 0, 0));
+      GTK_ADJUSTMENT(gtk_adjustment_new(0, 0, 50, 1, 0, 0));
 
   hscale = gtk_hscale_new(GTK_ADJUSTMENT(pdialog->factor[NUM_STATS]));
   gtk_table_attach_defaults(GTK_TABLE(table), hscale, 2, 3,
@@ -312,9 +311,9 @@ struct cma_dialog *create_cma_dialog(struct city *pcity, GtkAccelGroup *accel)
 
   gtk_widget_show_all(pdialog->shell);
 
-  ensure_initialised_cma_dialog_list();
+  ensure_initialised_dialog_list();
 
-  cma_dialog_list_insert(&cma_dialog_list, pdialog);
+  dialog_list_insert(&dialog_list, pdialog);
 
   update_cma_preset_list(pdialog);
 
@@ -328,8 +327,8 @@ struct cma_dialog *create_cma_dialog(struct city *pcity, GtkAccelGroup *accel)
 **************************************************************************/
 void refresh_cma_dialog(struct city *pcity, enum cma_refresh refresh)
 {
-  struct cma_result result;
-  struct cma_parameter param;
+  struct cm_result result;
+  struct cm_parameter param;
   struct cma_dialog *pdialog = get_cma_dialog(pcity);
   int controlled = cma_is_city_under_agent(pcity, NULL);
   int preset_index;
@@ -337,7 +336,7 @@ void refresh_cma_dialog(struct city *pcity, enum cma_refresh refresh)
   cmafec_get_fe_parameter(pcity, &param);
 
   /* fill in result label */
-  cma_query_result(pcity, &param, &result);
+  cm_copy_result_from_city(pcity, &result);
   gtk_set_label(pdialog->result_label,
 		(char *) cmafec_get_result_descr(pcity, &result, &param));
 
@@ -359,10 +358,13 @@ void refresh_cma_dialog(struct city *pcity, enum cma_refresh refresh)
   }
 
   gtk_widget_set_sensitive(GTK_WIDGET(pdialog->change_command),
+			   can_client_issue_orders() &&
 			   result.found_a_valid && !controlled);
   gtk_widget_set_sensitive(GTK_WIDGET(pdialog->perm_command),
+			   can_client_issue_orders() &&
 			   result.found_a_valid && !controlled);
   gtk_widget_set_sensitive(GTK_WIDGET(pdialog->release_command),
+			   can_client_issue_orders() &&
 			   controlled);
 }
 
@@ -389,7 +391,7 @@ static void update_cma_preset_list(struct cma_dialog *pdialog)
       gtk_clist_insert(GTK_CLIST(pdialog->preset_list), i, row);
     }
   } else {
-    static char *info_message_[4] = {
+    static const char *info_message_[4] = {
       N_("For information on:"),
       N_("CMA and presets"),
       N_("including sample presets,"),
@@ -419,7 +421,7 @@ static void cma_select_preset_callback(GtkWidget *w, gint row, gint col,
   struct cma_dialog *pdialog = (struct cma_dialog *) data;
   GList *selection = GTK_CLIST(pdialog->preset_list)->selection;
   int preset_index;
-  const struct cma_parameter *pparam;
+  const struct cm_parameter *pparam;
 
   if (!selection) {
     return;
@@ -496,7 +498,7 @@ static void cma_preset_add_callback_no(gpointer data)
 static void cma_preset_add_callback_yes(const char *input, gpointer data)
 {
   struct cma_dialog *pdialog = (struct cma_dialog *) data;
-  struct cma_parameter param;
+  struct cm_parameter param;
 
   cmafec_get_fe_parameter(pdialog->pcity, &param);
   cmafec_preset_add(input, &param);
@@ -597,12 +599,12 @@ static void cma_preset_remove_callback_yes(gpointer data)
 **************************************************************************/
 static void cma_change_to_callback(GtkWidget *w, gpointer data)
 {
-  struct cma_result result;
+  struct cm_result result;
   struct cma_dialog *pdialog = (struct cma_dialog *) data;
-  struct cma_parameter param;
+  struct cm_parameter param;
 
   cmafec_get_fe_parameter(pdialog->pcity, &param);
-  cma_query_result(pdialog->pcity, &param, &result);
+  cm_query_result(pdialog->pcity, &param, &result);
   cma_apply_result(pdialog->pcity, &result);
 }
 
@@ -613,7 +615,7 @@ static void cma_change_to_callback(GtkWidget *w, gpointer data)
 static void cma_change_permanent_to_callback(GtkWidget *w, gpointer data)
 {
   struct cma_dialog *pdialog = (struct cma_dialog *) data;
-  struct cma_parameter param;
+  struct cm_parameter param;
 
   cmafec_get_fe_parameter(pdialog->pcity, &param);
   cma_put_city_under_agent(pdialog->pcity, &param);
@@ -635,7 +637,7 @@ static void cma_release_callback(GtkWidget *w, gpointer data)
  called to adjust the sliders when a preset is selected
  notice that we don't want to call update_result here. 
 *****************************************************************/
-static void set_hscales(const struct cma_parameter *const parameter,
+static void set_hscales(const struct cm_parameter *const parameter,
 			struct cma_dialog *pdialog)
 {
   int i;
@@ -659,7 +661,7 @@ static void set_hscales(const struct cma_parameter *const parameter,
 static void hscale_changed(GtkAdjustment *get, gpointer data)
 {
   struct cma_dialog *pdialog = (struct cma_dialog *) data;
-  struct cma_parameter param;
+  struct cm_parameter param;
   int i;
 
   if (!allow_refreshes) {
@@ -671,7 +673,6 @@ static void hscale_changed(GtkAdjustment *get, gpointer data)
     param.minimal_surplus[i] = (int) (pdialog->minimal_surplus[i]->value);
     param.factor[i] = (int) (pdialog->factor[i]->value);
   }
-  param.factor_target = FT_SURPLUS;
   param.require_happy =
       (GTK_TOGGLE_BUTTON(pdialog->happy_button)->active ? 1 : 0);
   param.happy_factor = (int) (pdialog->factor[NUM_STATS]->value);
