@@ -10,19 +10,19 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 ***********************************************************************/
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <ctype.h>
 
 #include <gtk/gtk.h>
 
+#include "diptreaty.h"
 #include "fcintl.h"
 #include "game.h"
-#include "genlist.h"
 #include "government.h"
 #include "map.h"
 #include "mem.h"
@@ -31,10 +31,10 @@
 #include "shared.h"
 #include "support.h"
 
-#include "chatline.h"
 #include "climisc.h"
 #include "clinet.h"
-#include "diptreaty.h"
+
+#include "chatline.h"
 #include "gui_main.h"
 #include "gui_stuff.h"
 #include "mapview.h"
@@ -66,6 +66,8 @@ struct Diplomacy_dialog {
   GtkWidget *dip_pact_menu;
   GtkWidget *dip_vision_button0;
   GtkWidget *dip_vision_button1;
+  GtkWidget *dip_embassy_button0;
+  GtkWidget *dip_embassy_button1;
 
   GtkWidget *dip_label;
   GtkWidget *dip_clauselabel;
@@ -79,16 +81,24 @@ struct Diplomacy_dialog {
   GtkWidget *dip_erase_clause_command;
 };
 
-static struct genlist diplomacy_dialogs;
-static int diplomacy_dialogs_list_has_been_initialised;
+#define SPECLIST_TAG dialog
+#define SPECLIST_TYPE struct Diplomacy_dialog
+#include "speclist.h"
+
+#define dialog_list_iterate(dialoglist, pdialog) \
+    TYPED_LIST_ITERATE(struct Diplomacy_dialog, dialoglist, pdialog)
+#define dialog_list_iterate_end  LIST_ITERATE_END
+
+static struct dialog_list dialog_list;
+static bool dialog_list_list_has_been_initialised = FALSE;
 
 static struct Diplomacy_dialog *create_diplomacy_dialog(struct player *plr0, 
 						 struct player *plr1);
-
-static struct Diplomacy_dialog *find_diplomacy_dialog(struct player *plr0, 
-					       struct player *plr1);
-static void popup_diplomacy_dialog(struct player *plr0, struct player *plr1);
+static struct Diplomacy_dialog *find_diplomacy_dialog(int other_player_id);
+static void popup_diplomacy_dialog(int other_player_id);
 static void diplomacy_dialog_close_callback(GtkWidget *w, gpointer data);
+static gint diplomacy_dialog_delete_callback(GtkWidget * w, GdkEvent * ev,
+					     gpointer data);
 static void diplomacy_dialog_map_callback(GtkWidget *w, gpointer data);
 static void diplomacy_dialog_seamap_callback(GtkWidget *w, gpointer data);
 static void diplomacy_dialog_erase_clause_callback(GtkWidget *w, gpointer data);
@@ -99,6 +109,7 @@ static void diplomacy_dialog_ceasefire_callback(GtkWidget *w, gpointer data);
 static void diplomacy_dialog_peace_callback(GtkWidget *w, gpointer data);
 static void diplomacy_dialog_alliance_callback(GtkWidget *w, gpointer data);
 static void diplomacy_dialog_vision_callback(GtkWidget *w, gpointer data);
+static void diplomacy_dialog_embassy_callback(GtkWidget *w, gpointer data);
 static void close_diplomacy_dialog(struct Diplomacy_dialog *pdialog);
 static void update_diplomacy_dialog(struct Diplomacy_dialog *pdialog);
 static gint diplomacy_dialog_mbutton_callback(GtkWidget *w, GdkEvent *event);
@@ -107,93 +118,96 @@ static void diplo_dialog_returnkey(GtkWidget *w, gpointer data);
 /****************************************************************
 ...
 *****************************************************************/
-void handle_diplomacy_accept_treaty(struct packet_diplomacy_info *pa)
+void handle_diplomacy_accept_treaty(int counterpart, bool I_accepted,
+				    bool other_accepted)
 {
-  struct Diplomacy_dialog *pdialog;
-  
-  if((pdialog=find_diplomacy_dialog(&game.players[pa->plrno0],
-				&game.players[pa->plrno1]))) {
-    if(pa->plrno_from==game.player_idx)
-      pdialog->treaty.accept0=!pdialog->treaty.accept0;
-    else
-      pdialog->treaty.accept1=!pdialog->treaty.accept1;
-    update_diplomacy_dialog(pdialog);
-  }
-}
+  struct Diplomacy_dialog *pdialog = find_diplomacy_dialog(counterpart);
 
-/****************************************************************
-...
-*****************************************************************/
-void handle_diplomacy_init_meeting(struct packet_diplomacy_info *pa)
-{
-  popup_diplomacy_dialog(&game.players[pa->plrno0], 
-			 &game.players[pa->plrno1]);
-}
-
-/****************************************************************
-...
-*****************************************************************/
-void handle_diplomacy_cancel_meeting(struct packet_diplomacy_info *pa)
-{
-  struct Diplomacy_dialog *pdialog;
-  
-  if((pdialog=find_diplomacy_dialog(&game.players[pa->plrno0],
-				    &game.players[pa->plrno1])))
-    close_diplomacy_dialog(pdialog);
-}
-
-/****************************************************************
-...
-*****************************************************************/
-void handle_diplomacy_create_clause(struct packet_diplomacy_info *pa)
-{
-  struct Diplomacy_dialog *pdialog;
-  
-  if((pdialog=find_diplomacy_dialog(&game.players[pa->plrno0],
-				&game.players[pa->plrno1]))) {
-    add_clause(&pdialog->treaty, &game.players[pa->plrno_from],
-	       pa->clause_type, pa->value);
-    update_diplomacy_dialog(pdialog);
+  if (!pdialog) {
+    return;
   }
 
+  pdialog->treaty.accept0 = I_accepted;
+  pdialog->treaty.accept1 = other_accepted;
+
+  update_diplomacy_dialog(pdialog);
 }
 
 /****************************************************************
 ...
 *****************************************************************/
-void handle_diplomacy_remove_clause(struct packet_diplomacy_info *pa)
+void handle_diplomacy_init_meeting(int counterpart, int initiated_from)
 {
-  struct Diplomacy_dialog *pdialog;
-
-  if((pdialog=find_diplomacy_dialog(&game.players[pa->plrno0],
-				&game.players[pa->plrno1]))) {
-    remove_clause(&pdialog->treaty, &game.players[pa->plrno_from],
-		  pa->clause_type, pa->value);
-    
-    update_diplomacy_dialog(pdialog);
-  }
-
+  popup_diplomacy_dialog(counterpart);
 }
 
+/****************************************************************
+...
+*****************************************************************/
+void handle_diplomacy_cancel_meeting(int counterpart, int initiated_from)
+{
+  struct Diplomacy_dialog *pdialog = find_diplomacy_dialog(counterpart);
 
+  if (!pdialog) {
+    return;
+  }
 
+  close_diplomacy_dialog(pdialog);
+}
+
+/****************************************************************
+...
+*****************************************************************/
+void handle_diplomacy_create_clause(int counterpart, int giver,
+				    enum clause_type type, int value)
+{
+  struct Diplomacy_dialog *pdialog = find_diplomacy_dialog(counterpart);
+
+  if (!pdialog) {
+    return;
+  }
+
+  add_clause(&pdialog->treaty, get_player(giver), type, value);
+  update_diplomacy_dialog(pdialog);
+}
+
+/****************************************************************
+...
+*****************************************************************/
+void handle_diplomacy_remove_clause(int counterpart, int giver,
+				    enum clause_type type, int value)
+{
+  struct Diplomacy_dialog *pdialog = find_diplomacy_dialog(counterpart);
+
+  if (!pdialog) {
+    return;
+  }
+
+  remove_clause(&pdialog->treaty, get_player(giver), type, value);
+  update_diplomacy_dialog(pdialog);
+}
 
 /****************************************************************
 popup the dialog 10% inside the main-window 
 *****************************************************************/
-static void popup_diplomacy_dialog(struct player *plr0, struct player *plr1)
+static void popup_diplomacy_dialog(int other_player_id)
 {
-  struct Diplomacy_dialog *pdialog;
-  
-  if(!(pdialog=find_diplomacy_dialog(plr0, plr1))) {
-    pdialog=create_diplomacy_dialog(plr0, plr1);
-    gtk_set_relative_position(toplevel,pdialog->dip_dialog_shell,0,0);
+  struct Diplomacy_dialog *pdialog = find_diplomacy_dialog(other_player_id);
+
+  if (game.player_ptr->ai.control) {
+    return;			/* Don't show if we are AI controlled. */
+  }
+
+  if (!pdialog) {
+    pdialog =
+	create_diplomacy_dialog(game.player_ptr,
+				get_player(other_player_id));
+    gtk_set_relative_position(toplevel, pdialog->dip_dialog_shell, 0, 0);
   }
 
   gtk_widget_show(pdialog->dip_dialog_shell);
 }
 
-
 /****************************************************************
 ...
 *****************************************************************/
@@ -203,16 +217,18 @@ static int fill_diplomacy_tech_menu(GtkWidget *popupmenu,
   int i, flag;
 
   for(i=1, flag=0; i<game.num_tech_types; i++) {
-    if(get_invention(plr0, i)==TECH_KNOWN && 
-       (get_invention(plr1, i)==TECH_UNKNOWN || 
-	get_invention(plr1, i)==TECH_REACHABLE))
-	{
+    if (get_invention(plr0, i) == TECH_KNOWN
+        && (get_invention(plr1, i) == TECH_UNKNOWN
+	    || get_invention(plr1, i) == TECH_REACHABLE)
+        && tech_is_available(plr1, i)) {
 	  GtkWidget *item=gtk_menu_item_new_with_label(advances[i].name);
 
 	  gtk_menu_append(GTK_MENU(popupmenu),item);
-	  gtk_signal_connect(GTK_OBJECT(item),"activate",
-	    GTK_SIGNAL_FUNC(diplomacy_dialog_tech_callback),
-	    (gpointer)(plr0->player_no*10000+plr1->player_no*100+i));
+	  gtk_signal_connect(GTK_OBJECT(item), "activate",
+			     GTK_SIGNAL_FUNC(diplomacy_dialog_tech_callback),
+			 GINT_TO_POINTER((plr0->player_no << 24) |
+					 (plr1->player_no << 16) |
+					 i));
       flag=1;
     }
   }
@@ -227,33 +243,38 @@ cities visible to requesting player.
 
 			    - Kris Bubendorfer
 *****************************************************************/
-static int fill_diplomacy_city_menu(GtkWidget *popupmenu, 
-				    struct player *plr0, struct player *plr1)
+static int fill_diplomacy_city_menu(GtkWidget * popupmenu,
+				    struct player *pgiver,
+				    struct player *pdest)
 {
-  int i = 0, j = 0, n = city_list_size(&plr0->cities);
+  int i = 0, j = 0, n = city_list_size(&pgiver->cities);
   struct city **city_list_ptrs;
-  if (n>0) {
-    city_list_ptrs = fc_malloc(sizeof(struct city*)*n);
+
+  if (n > 0) {
+    city_list_ptrs = fc_malloc(sizeof(struct city *) * n);
   } else {
     city_list_ptrs = NULL;
   }
 
-  city_list_iterate(plr0->cities, pcity) {
-    if(!city_got_effect(pcity, B_PALACE)){
+  city_list_iterate(pgiver->cities, pcity) {
+    if (!is_capital(pcity)) {
       city_list_ptrs[i] = pcity;
       i++;
     }
   } city_list_iterate_end;
 
-  qsort(city_list_ptrs, i, sizeof(struct city*), city_name_compare);
+  qsort(city_list_ptrs, i, sizeof(struct city *), city_name_compare);
   
-  for(j=0; j<i; j++) {
-      GtkWidget *item=gtk_menu_item_new_with_label(city_list_ptrs[j]->name);
+  for (j = 0; j < i; j++) {
+      GtkWidget *item =
+	  gtk_menu_item_new_with_label(city_list_ptrs[j]->name);
 
       gtk_menu_append(GTK_MENU(popupmenu),item);
-      gtk_signal_connect(GTK_OBJECT(item),"activate",
-        GTK_SIGNAL_FUNC(diplomacy_dialog_city_callback),
-        (gpointer)(city_list_ptrs[j]->id*1024 + plr0->player_no*32 + plr1->player_no));
+      gtk_signal_connect(GTK_OBJECT(item), "activate",
+			 GTK_SIGNAL_FUNC(diplomacy_dialog_city_callback),
+			 GINT_TO_POINTER((pgiver->player_no << 24) |
+					 (pdest->player_no << 16) |
+					 city_list_ptrs[j]->id));
   }
   free(city_list_ptrs);
   return i;
@@ -266,8 +287,8 @@ static struct Diplomacy_dialog *create_diplomacy_dialog(struct player *plr0,
 							struct player *plr1)
 {
   char buf[512];
-  static gchar *titles_[1]
-    = { N_("The following clauses have been agreed upon:") };
+  static const char *titles_[1]
+      = { N_("The following clauses have been agreed upon:") };
   static gchar **titles;
   struct Diplomacy_dialog *pdialog;
   GtkWidget *button,*label,*item,*table,*scrolled;
@@ -275,7 +296,7 @@ static struct Diplomacy_dialog *create_diplomacy_dialog(struct player *plr0,
   if (!titles) titles = intl_slist(1, titles_);
   
   pdialog=fc_malloc(sizeof(struct Diplomacy_dialog));
-  genlist_insert(&diplomacy_dialogs, pdialog, 0);
+  dialog_list_insert(&dialog_list, pdialog);
   
   init_treaty(&pdialog->treaty, plr0, plr1);
   
@@ -423,7 +444,7 @@ static struct Diplomacy_dialog *create_diplomacy_dialog(struct player *plr0,
   gtk_signal_connect(GTK_OBJECT(pdialog->dip_vision_button0), "clicked",
 		     GTK_SIGNAL_FUNC(diplomacy_dialog_vision_callback),
 		     (gpointer)pdialog);
-  if (plr0->gives_shared_vision & (1<<plr1->player_no))
+  if (gives_shared_vision(plr0, plr1))
     gtk_widget_set_sensitive(pdialog->dip_vision_button0, FALSE);
 
   pdialog->dip_vision_button1=gtk_button_new_with_label(_("Give shared vision"));
@@ -432,9 +453,31 @@ static struct Diplomacy_dialog *create_diplomacy_dialog(struct player *plr0,
   gtk_signal_connect(GTK_OBJECT(pdialog->dip_vision_button1), "clicked",
 		     GTK_SIGNAL_FUNC(diplomacy_dialog_vision_callback),
 		     (gpointer)pdialog);
-  if (plr1->gives_shared_vision & (1<<plr0->player_no))
+  if (gives_shared_vision(plr1, plr0))
     gtk_widget_set_sensitive(pdialog->dip_vision_button1, FALSE);
 
+  pdialog->dip_embassy_button0 = gtk_button_new_with_label(_("Give embassy"));
+  gtk_box_pack_start(GTK_BOX(pdialog->dip_vbox0),
+		     pdialog->dip_embassy_button0,
+		     FALSE, FALSE, 2);
+  gtk_signal_connect(GTK_OBJECT(pdialog->dip_embassy_button0), "clicked",
+                     GTK_SIGNAL_FUNC(diplomacy_dialog_embassy_callback),
+		     (gpointer)pdialog);
+  if (player_has_embassy(plr1, plr0)) {
+    gtk_widget_set_sensitive(pdialog->dip_embassy_button0, FALSE);
+  }
+
+  pdialog->dip_embassy_button1 = gtk_button_new_with_label(_("Give embassy"));
+  gtk_box_pack_start(GTK_BOX(pdialog->dip_vbox1),
+		     pdialog->dip_embassy_button1,
+		     FALSE, FALSE, 2);
+  gtk_signal_connect(GTK_OBJECT(pdialog->dip_embassy_button1), "clicked",
+                     GTK_SIGNAL_FUNC(diplomacy_dialog_embassy_callback),
+		     (gpointer)pdialog);
+  if (player_has_embassy(plr0, plr1)) {
+    gtk_widget_set_sensitive(pdialog->dip_embassy_button1, FALSE);
+  }
+
   /* Start of pact button insertion */
 
   pdialog->dip_pact_menu=gtk_menu_new();
@@ -526,6 +569,10 @@ static struct Diplomacy_dialog *create_diplomacy_dialog(struct player *plr0,
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(pdialog->dip_dialog_shell)->action_area),
 	pdialog->dip_close_command, TRUE, TRUE, 2 );
 
+  gtk_signal_connect(GTK_OBJECT(pdialog->dip_dialog_shell), "delete_event",
+		     GTK_SIGNAL_FUNC(diplomacy_dialog_delete_callback),
+		     pdialog);
+
   gtk_signal_connect(GTK_OBJECT(pdialog->dip_close_command), "clicked",
 	GTK_SIGNAL_FUNC(diplomacy_dialog_close_callback), pdialog);
 
@@ -556,24 +603,18 @@ static struct Diplomacy_dialog *create_diplomacy_dialog(struct player *plr0,
 **************************************************************************/
 static void update_diplomacy_dialog(struct Diplomacy_dialog *pdialog)
 {
-         int               i;
          char              buf		[64];
   static char             *row		[1];
-  struct genlist_iterator  myiter;
   
   row[0]=buf;
 
   gtk_clist_freeze(GTK_CLIST(pdialog->dip_clauselist));
   gtk_clist_clear(GTK_CLIST(pdialog->dip_clauselist));
 
-  genlist_iterator_init(&myiter, &pdialog->treaty.clauses, 0);
-  
-  for(i=0; i<MAX_NUM_CLAUSES && ITERATOR_PTR(myiter); ITERATOR_NEXT(myiter)) {
-    struct Clause *pclause=(struct Clause *)ITERATOR_PTR(myiter);
+  clause_list_iterate(pdialog->treaty.clauses, pclause) {
     client_diplomacy_clause_string(buf, sizeof(buf), pclause);
     gtk_clist_append(GTK_CLIST(pdialog->dip_clauselist),row);
-    i++;
-  }
+  } clause_list_iterate_end;
 
   gtk_clist_thaw(GTK_CLIST(pdialog->dip_clauselist));
   gtk_widget_show_all(pdialog->dip_clauselist);
@@ -587,48 +628,42 @@ static void update_diplomacy_dialog(struct Diplomacy_dialog *pdialog)
 /****************************************************************
 ...
 *****************************************************************/
-static void diplomacy_dialog_tech_callback(GtkWidget *w, gpointer data)
+static void diplomacy_dialog_tech_callback(GtkWidget * w, gpointer data)
 {
-  size_t choice;
-  struct packet_diplomacy_info pa;
-  
-  choice=(size_t)data;
+  int choice = GPOINTER_TO_INT(data);
+  int giver = (choice >> 24) & 0xff, dest = (choice >> 16) & 0xff, other;
+  int tech = choice & 0xffff;
 
-  pa.plrno0=choice/10000;
-  pa.plrno1=(choice/100)%100;
-  pa.clause_type=CLAUSE_ADVANCE;
-  pa.plrno_from=pa.plrno0;
-  pa.value=choice%100;
-    
-  send_packet_diplomacy_info(&aconnection, PACKET_DIPLOMACY_CREATE_CLAUSE,
-			    &pa);
+  if (giver == game.player_idx) {
+    other = dest;
+  } else {
+    other = giver;
+  }
+
+  dsend_packet_diplomacy_create_clause_req(&aconnection, other, giver,
+					   CLAUSE_ADVANCE, tech);
 }
 
 /****************************************************************
 Callback for trading cities
 			      - Kris Bubendorfer
 *****************************************************************/
-static void diplomacy_dialog_city_callback(GtkWidget *w, gpointer data)
+static void diplomacy_dialog_city_callback(GtkWidget * w, gpointer data)
 {
-  size_t choice;
-  struct packet_diplomacy_info pa;
-  
-  choice=(size_t)data;
+  int choice = GPOINTER_TO_INT(data);
+  int giver = (choice >> 24) & 0xff, dest = (choice >> 16) & 0xff, other;
+  int city = choice & 0xffff;
 
-  pa.value = choice/1024;
-  choice -= pa.value * 1024;
-  pa.plrno0 = choice/32;
-  choice -= pa.plrno0 * 32;
-  pa.plrno1 = choice;
- 
-  pa.clause_type=CLAUSE_CITY;
-  pa.plrno_from=pa.plrno0;
+  if (giver == game.player_idx) {
+    other = dest;
+  } else {
+    other = giver;
+  }
 
-  send_packet_diplomacy_info(&aconnection, PACKET_DIPLOMACY_CREATE_CLAUSE,
-			     &pa);
+  dsend_packet_diplomacy_create_clause_req(&aconnection, other, giver,
+					   CLAUSE_CITY, city);
 }
 
-
 /****************************************************************
 ...
 *****************************************************************/
@@ -638,34 +673,25 @@ static void diplomacy_dialog_erase_clause_callback(GtkWidget *w, gpointer data)
   GList              *selection;
 
   if ( (selection=GTK_CLIST(pdialog->dip_clauselist)->selection) ) {
-    int i,row;
-    struct genlist_iterator myiter;
+    int i = 0, row;
     
-    row = (int)selection->data;
+    row = GPOINTER_TO_INT(selection->data);
   
-    genlist_iterator_init(&myiter, &pdialog->treaty.clauses, 0);
-
-    for(i=0; ITERATOR_PTR(myiter); ITERATOR_NEXT(myiter), i++) {
-      if(i==row) {
-	struct packet_diplomacy_info pa;
-	struct Clause *pclause=(struct Clause *)ITERATOR_PTR(myiter);
-
-	pa.plrno0=pdialog->treaty.plr0->player_no;
-	pa.plrno1=pdialog->treaty.plr1->player_no;
-	pa.plrno_from=pclause->from->player_no;
-	pa.clause_type=pclause->type;
-	pa.value=pclause->value;
-	send_packet_diplomacy_info(&aconnection, PACKET_DIPLOMACY_REMOVE_CLAUSE,
-				   &pa);
+    clause_list_iterate(pdialog->treaty.clauses, pclause) {
+      if (i == row) {
+	dsend_packet_diplomacy_remove_clause_req(&aconnection,
+						 pdialog->treaty.plr1->
+						 player_no,
+						 pclause->from->player_no,
+						 pclause->type,
+						 pclause->value);
 	return;
       }
-    }
+      i++;
+    } clause_list_iterate_end;
   }
 }
 
-
-
-
 /****************************************************************
 ...
 *****************************************************************/
@@ -686,20 +712,13 @@ static gint diplomacy_dialog_mbutton_callback(GtkWidget *w, GdkEvent *event)
 *****************************************************************/
 static void diplomacy_dialog_map_callback(GtkWidget *w, gpointer data)
 {
-  struct Diplomacy_dialog *pdialog=(struct Diplomacy_dialog *)data;
-  struct packet_diplomacy_info pa;
-  struct player *pgiver;
+  struct Diplomacy_dialog *pdialog = (struct Diplomacy_dialog *) data;
+  struct player *pgiver = ((w->parent) == pdialog->dip_map_menu0) ?
+      pdialog->treaty.plr0 : pdialog->treaty.plr1;
   
-  pgiver=((w->parent)==pdialog->dip_map_menu0) ? 
-    pdialog->treaty.plr0 : pdialog->treaty.plr1;
-  
-  pa.plrno0=pdialog->treaty.plr0->player_no;
-  pa.plrno1=pdialog->treaty.plr1->player_no;
-  pa.clause_type=CLAUSE_MAP;
-  pa.plrno_from=pgiver->player_no;
-  pa.value=0;
-  send_packet_diplomacy_info(&aconnection, PACKET_DIPLOMACY_CREATE_CLAUSE,
-			     &pa);
+  dsend_packet_diplomacy_create_clause_req(&aconnection,
+					   pdialog->treaty.plr1->player_no,
+					   pgiver->player_no, CLAUSE_MAP, 0);
 }
 
 /****************************************************************
@@ -707,38 +726,29 @@ static void diplomacy_dialog_map_callback(GtkWidget *w, gpointer data)
 *****************************************************************/
 static void diplomacy_dialog_seamap_callback(GtkWidget *w, gpointer data)
 {
-  struct Diplomacy_dialog *pdialog=(struct Diplomacy_dialog *)data;
-  struct packet_diplomacy_info pa;
-  struct player *pgiver;
-  
-  pgiver=((w->parent)==pdialog->dip_map_menu0) ? 
-    pdialog->treaty.plr0 : pdialog->treaty.plr1;
-  
-  pa.plrno0=pdialog->treaty.plr0->player_no;
-  pa.plrno1=pdialog->treaty.plr1->player_no;
-  pa.clause_type=CLAUSE_SEAMAP;
-  pa.plrno_from=pgiver->player_no;
-  pa.value=0;
-  send_packet_diplomacy_info(&aconnection, PACKET_DIPLOMACY_CREATE_CLAUSE,
-			     &pa);
+  struct Diplomacy_dialog *pdialog = (struct Diplomacy_dialog *) data;
+  struct player *pgiver = ((w->parent) == pdialog->dip_map_menu0) ?
+      pdialog->treaty.plr0 : pdialog->treaty.plr1;
+
+  dsend_packet_diplomacy_create_clause_req(&aconnection,
+					   pdialog->treaty.plr1->player_no,
+					   pgiver->player_no, CLAUSE_SEAMAP,
+					   0);
 }
 
 /****************************************************************
 ...
 *****************************************************************/
-static void diplomacy_dialog_add_pact_clause(GtkWidget *w, gpointer data,
-						    int type)
+static void diplomacy_dialog_add_pact_clause(GtkWidget * w, gpointer data,
+					     int type)
 {
-  struct Diplomacy_dialog *pdialog=(struct Diplomacy_dialog *)data;
-  struct packet_diplomacy_info pa;
+  struct Diplomacy_dialog *pdialog = (struct Diplomacy_dialog *) data;
+  struct player *pgiver = ((w->parent) == pdialog->dip_map_menu0) ?
+      pdialog->treaty.plr0 : pdialog->treaty.plr1;
   
-  pa.plrno0 = pdialog->treaty.plr0->player_no;
-  pa.plrno1 = pdialog->treaty.plr1->player_no;
-  pa.clause_type = type;
-  pa.plrno_from = pdialog->treaty.plr0->player_no;
-  pa.value = 0;
-  send_packet_diplomacy_info(&aconnection, PACKET_DIPLOMACY_CREATE_CLAUSE,
-			     &pa);
+  dsend_packet_diplomacy_create_clause_req(&aconnection,
+					   pdialog->treaty.plr1->player_no,
+					   pgiver->player_no, type, 0);
 }
 
 /****************************************************************
@@ -771,38 +781,51 @@ static void diplomacy_dialog_alliance_callback(GtkWidget *w, gpointer data)
 static void diplomacy_dialog_vision_callback(GtkWidget *w, gpointer data)
 {
   struct Diplomacy_dialog *pdialog=(struct Diplomacy_dialog *)data;
-  struct packet_diplomacy_info pa;
-  struct player *pgiver;
+  struct player *pgiver = (w == pdialog->dip_vision_button0) ?
+      pdialog->treaty.plr0 : pdialog->treaty.plr1;
 
-  pgiver = (w == pdialog->dip_vision_button0) ? 
-    pdialog->treaty.plr0 : pdialog->treaty.plr1;
-
-  pa.plrno0 = pdialog->treaty.plr0->player_no;
-  pa.plrno1 = pdialog->treaty.plr1->player_no;
-  pa.clause_type = CLAUSE_VISION;
-  pa.plrno_from = pgiver->player_no;
-  pa.value = 0;
-  send_packet_diplomacy_info(&aconnection, PACKET_DIPLOMACY_CREATE_CLAUSE,
-			     &pa);
+  dsend_packet_diplomacy_create_clause_req(&aconnection,
+					   pdialog->treaty.plr1->player_no,
+					   pgiver->player_no, CLAUSE_VISION,
+					   0);
 }
 
+/****************************************************************
+  Called whenever give embassy button is clicked
+*****************************************************************/
+static void diplomacy_dialog_embassy_callback(GtkWidget *w, gpointer data)
+{
+  struct Diplomacy_dialog *pdialog=(struct Diplomacy_dialog *)data;    
+  struct player *pgiver = (w == pdialog->dip_embassy_button0) ?
+      pdialog->treaty.plr0 : pdialog->treaty.plr1;
+  dsend_packet_diplomacy_create_clause_req(&aconnection,
+					   pdialog->treaty.plr1->player_no,
+					   pgiver->player_no, CLAUSE_EMBASSY,
+					   0);
+}
 
 /****************************************************************
 ...
 *****************************************************************/
 static void diplomacy_dialog_close_callback(GtkWidget *w, gpointer data)
 {
-  struct Diplomacy_dialog *pdialog=(struct Diplomacy_dialog *)data;
-  struct packet_diplomacy_info pa;
+  struct Diplomacy_dialog *pdialog = (struct Diplomacy_dialog *) data;
 
-  pa.plrno0=game.player_idx;
-  pa.plrno1=pdialog->treaty.plr1->player_no;
-  send_packet_diplomacy_info(&aconnection, PACKET_DIPLOMACY_CANCEL_MEETING, 
-			     &pa);
+  dsend_packet_diplomacy_cancel_meeting_req(&aconnection,
+					    pdialog->treaty.plr1->player_no);
   
   close_diplomacy_dialog(pdialog);
 }
 
+/****************************************************************
+...
+*****************************************************************/
+static gint diplomacy_dialog_delete_callback(GtkWidget * w, GdkEvent * ev,
+					     gpointer data)
+{
+  diplomacy_dialog_close_callback(NULL, data);
+  return FALSE;
+}
 
 /****************************************************************
 ...
@@ -810,13 +833,9 @@ static void diplomacy_dialog_close_callback(GtkWidget *w, gpointer data)
 static void diplomacy_dialog_accept_callback(GtkWidget *w, gpointer data)
 {
   struct Diplomacy_dialog *pdialog=(struct Diplomacy_dialog *)data;
-  struct packet_diplomacy_info pa;
-  
-  pa.plrno0=pdialog->treaty.plr0->player_no;
-  pa.plrno1=pdialog->treaty.plr1->player_no;
-  pa.plrno_from=game.player_idx;
-  send_packet_diplomacy_info(&aconnection, PACKET_DIPLOMACY_ACCEPT_TREATY,
-			     &pa);
+
+  dsend_packet_diplomacy_accept_treaty_req(&aconnection,
+					   pdialog->treaty.plr1->player_no);
 }
 
 
@@ -827,33 +846,30 @@ void close_diplomacy_dialog(struct Diplomacy_dialog *pdialog)
 {
   gtk_widget_destroy(pdialog->dip_dialog_shell);
   
-  genlist_unlink(&diplomacy_dialogs, pdialog);
+  dialog_list_unlink(&dialog_list, pdialog);
   free(pdialog);
 }
 
 /*****************************************************************
 ...
 *****************************************************************/
-static struct Diplomacy_dialog *find_diplomacy_dialog(struct player *plr0, 
-						      struct player *plr1)
+static struct Diplomacy_dialog *find_diplomacy_dialog(int other_player_id)
 {
-  struct genlist_iterator myiter;
+  struct player *plr0 = game.player_ptr, *plr1 = get_player(other_player_id);
 
-  if(!diplomacy_dialogs_list_has_been_initialised) {
-    genlist_init(&diplomacy_dialogs);
-    diplomacy_dialogs_list_has_been_initialised=1;
+  if (!dialog_list_list_has_been_initialised) {
+    dialog_list_init(&dialog_list);
+    dialog_list_list_has_been_initialised = TRUE;
   }
   
-  genlist_iterator_init(&myiter, &diplomacy_dialogs, 0);
-    
-  for(; ITERATOR_PTR(myiter); ITERATOR_NEXT(myiter)) {
-    struct Diplomacy_dialog *pdialog=
-      (struct Diplomacy_dialog *)ITERATOR_PTR(myiter);
-    if((pdialog->treaty.plr0==plr0 && pdialog->treaty.plr1==plr1) ||
-       (pdialog->treaty.plr0==plr1 && pdialog->treaty.plr1==plr0))
+  dialog_list_iterate(dialog_list, pdialog) {
+    if ((pdialog->treaty.plr0 == plr0 && pdialog->treaty.plr1 == plr1) ||
+	(pdialog->treaty.plr0 == plr1 && pdialog->treaty.plr1 == plr0)) {
       return pdialog;
-  }
-  return 0;
+    }
+  } dialog_list_iterate_end;
+
+  return NULL;
 }
 
 /*****************************************************************
@@ -861,18 +877,13 @@ static struct Diplomacy_dialog *find_diplomacy_dialog(struct player *plr0,
 *****************************************************************/
 static struct Diplomacy_dialog *find_diplomacy_by_input(GtkWidget *w)
 {
-  struct genlist_iterator myiter;
-  
-  genlist_iterator_init(&myiter, &diplomacy_dialogs, 0);
-    
-  for(; ITERATOR_PTR(myiter); ITERATOR_NEXT(myiter)) {
-    struct Diplomacy_dialog *pdialog=
-      (struct Diplomacy_dialog *)ITERATOR_PTR(myiter);
-    if((pdialog->dip_gold_entry0==w) || (pdialog->dip_gold_entry1==w)) {
+  dialog_list_iterate(dialog_list, pdialog) {
+    if ((pdialog->dip_gold_entry0 == w) || (pdialog->dip_gold_entry1 == w)) {
       return pdialog;
     }
-  }
-  return 0;
+  } dialog_list_iterate_end;
+
+  return NULL;
 }
 
 /*****************************************************************
@@ -880,32 +891,24 @@ static struct Diplomacy_dialog *find_diplomacy_by_input(GtkWidget *w)
 *****************************************************************/
 static void diplo_dialog_returnkey(GtkWidget *w, gpointer data)
 {
-  struct Diplomacy_dialog *pdialog;
+  struct Diplomacy_dialog *pdialog = find_diplomacy_by_input(w);
   
-  if((pdialog=find_diplomacy_by_input(w))) {
-    struct player *pgiver;
-    char *dp;
+  if (pdialog) {
+    struct player *pgiver = (w == pdialog->dip_gold_entry0) ?
+	pdialog->treaty.plr0 : pdialog->treaty.plr1;
+    char *dp = gtk_entry_get_text(GTK_ENTRY(w));
     int amount;
     
-    pgiver=(w==pdialog->dip_gold_entry0) ? 
-      pdialog->treaty.plr0 : pdialog->treaty.plr1;
-    
-    dp=gtk_entry_get_text(GTK_ENTRY(w));
-    amount=atoi(dp);
-    
-    if(amount>=0 && amount<=pgiver->economic.gold) {
-      struct packet_diplomacy_info pa;
-      pa.plrno0=pdialog->treaty.plr0->player_no;
-      pa.plrno1=pdialog->treaty.plr1->player_no;
-      pa.clause_type=CLAUSE_GOLD;
-      pa.plrno_from=pgiver->player_no;
-      pa.value=amount;
-      send_packet_diplomacy_info(&aconnection, PACKET_DIPLOMACY_CREATE_CLAUSE,
-				 &pa);
-      gtk_entry_set_text(GTK_ENTRY(w),"");
-    }
-    else
+    if (sscanf(dp, "%d", &amount) == 1 && amount >= 0
+	&& amount <= pgiver->economic.gold) {
+      dsend_packet_diplomacy_create_clause_req(&aconnection,
+					       pdialog->treaty.plr1->
+					       player_no, pgiver->player_no,
+					       CLAUSE_GOLD, amount);
+      gtk_entry_set_text(GTK_ENTRY(w), "");
+    } else {
       append_output_window(_("Game: Invalid amount of gold specified."));
+    }
   }
 }
 
@@ -914,13 +917,10 @@ static void diplo_dialog_returnkey(GtkWidget *w, gpointer data)
 *****************************************************************/
 void close_all_diplomacy_dialogs(void)
 {
-  struct Diplomacy_dialog *pdialog;
-  
-  if (!diplomacy_dialogs_list_has_been_initialised) {
+  if (!dialog_list_list_has_been_initialised) {
     return;
   }
-  while (genlist_size(&diplomacy_dialogs)) {
-    pdialog = genlist_get(&diplomacy_dialogs, 0);
-    close_diplomacy_dialog(pdialog);
+  while (dialog_list_size(&dialog_list) > 0) {
+    close_diplomacy_dialog(dialog_list_get(&dialog_list, 0));
   }
 }
