@@ -10,6 +10,7 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 ***********************************************************************/
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -18,8 +19,8 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
+#include <gtk/gtk.h>
 
 #include "events.h"
 #include "fcintl.h"
@@ -30,16 +31,17 @@
 #include "shared.h"
 #include "support.h"
 
+#include "clinet.h"
+#include "options.h"
+
 #include "chatline.h"
 #include "cityrep.h"
 #include "dialogs.h"
-#include "clinet.h"
 #include "gui_main.h"
 #include "gui_stuff.h"
-#include "options.h"
 
-#include "ratesdlg.h"
 #include "optiondlg.h"
+#include "ratesdlg.h"
 
 /******************************************************************/
 static GtkWidget *rates_dialog_shell;
@@ -195,15 +197,11 @@ static void rates_changed_callback(GtkAdjustment *adj)
 **************************************************************************/
 static void rates_ok_command_callback(GtkWidget *widget, gpointer data)
 {
-  struct packet_player_request packet;
-  
   gtk_widget_set_sensitive(top_vbox, TRUE);
   gtk_widget_destroy(rates_dialog_shell);
 
-  packet.tax=rates_tax_value;
-  packet.science=rates_sci_value;
-  packet.luxury=rates_lux_value;
-  send_packet_player_request(&aconnection, &packet, PACKET_PLAYER_RATES);
+  dsend_packet_player_rates(&aconnection, rates_tax_value, rates_lux_value,
+			    rates_sci_value);
 }
 
 
@@ -374,24 +372,37 @@ static GtkWidget *option_dialog_shell;
 **************************************************************************/
 static void option_ok_command_callback(GtkWidget *widget, gpointer data)
 {
-  client_option *o;
   char *dp;
+  bool b;
+  int val;
 
-  for (o=options; o->name; ++o) {
+  client_options_iterate(o) {
     switch (o->type) {
     case COT_BOOL:
+      b = *(o->p_bool_value);
       *(o->p_bool_value) = GTK_TOGGLE_BUTTON(o->p_gui_data)->active;
+      if (b != *(o->p_bool_value) && o->change_callback) {
+	(o->change_callback)(o);
+      }
       break;
     case COT_INT:
+      val = *(o->p_int_value);
       dp = gtk_entry_get_text(GTK_ENTRY(o->p_gui_data));
       sscanf(dp, "%d", o->p_int_value);
+      if (val != *(o->p_int_value) && o->change_callback) {
+	(o->change_callback)(o);
+      }
       break;
     case COT_STR:
       if (o->p_string_vals) {
-	mystrlcpy(o->p_string_value,
-		  gtk_entry_get_text(GTK_ENTRY
-				     (GTK_COMBO(o->p_gui_data)->entry)),
-		  o->string_length);
+	char* new_value = gtk_entry_get_text(GTK_ENTRY
+					    (GTK_COMBO(o->p_gui_data)->entry));
+	if (strcmp(o->p_string_value, new_value)) {
+	  mystrlcpy(o->p_string_value, new_value, o->string_length);
+	  if (o->change_callback) {
+	    (o->change_callback)(o);
+	  }
+	}
       } else {
 	mystrlcpy(o->p_string_value,
 		  gtk_entry_get_text(GTK_ENTRY(o->p_gui_data)),
@@ -399,7 +410,7 @@ static void option_ok_command_callback(GtkWidget *widget, gpointer data)
       }
       break;
     }
-  }
+  } client_options_iterate_end;
 
   gtk_widget_set_sensitive(top_vbox, TRUE);
   gtk_widget_destroy(option_dialog_shell);
@@ -421,7 +432,6 @@ static void create_option_dialog(void)
 {
   GtkWidget *button, *label, *table;
   GtkAccelGroup *accel=gtk_accel_group_new();
-  client_option *o;
   int i;
 
   option_dialog_shell = gtk_dialog_new();
@@ -434,14 +444,12 @@ static void create_option_dialog(void)
 
   gtk_window_set_title(GTK_WINDOW(option_dialog_shell), _("Set local options"));
 
-  for (o=options, i=0; o->name; ++o, i++)
-    ;
-
-  table=gtk_table_new(i, 2, FALSE);
+  table=gtk_table_new(num_options, 2, FALSE);
   gtk_container_add(GTK_CONTAINER(GTK_DIALOG(option_dialog_shell)->vbox),
   	table);
 
-  for (o=options, i=0; o->name; ++o, i++) {
+  i = 0;
+  client_options_iterate(o) {
     switch (o->type) {
     case COT_BOOL:
       label = gtk_label_new(_(o->description));
@@ -471,7 +479,8 @@ static void create_option_dialog(void)
       gtk_table_attach_defaults(GTK_TABLE(table), o->p_gui_data, 1, 2, i, i+1);
       break;
     }
-  }
+    i++;
+  } client_options_iterate_end;
 
   button = gtk_button_new_with_label( _("Close") );
   gtk_box_pack_start( GTK_BOX( GTK_DIALOG( option_dialog_shell )->action_area ),
@@ -492,12 +501,11 @@ static void create_option_dialog(void)
 *****************************************************************/
 void popup_option_dialog(void)
 {
-  client_option *o;
   char valstr[64];
 
   create_option_dialog();
 
-  for (o=options; o->name; ++o) {
+  client_options_iterate(o) {
     switch (o->type) {
     case COT_BOOL:
       gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(o->p_gui_data),
@@ -526,7 +534,7 @@ void popup_option_dialog(void)
       }
       break;
     }
-  }
+  } client_options_iterate_end;
 
 /*  gtk_set_relative_position(toplevel, option_dialog_shell, 25, 25);*/
   gtk_widget_show(option_dialog_shell);
