@@ -33,6 +33,10 @@
  * that we no longer need to insert it inside a GtkEventBox. -vasc
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include "gtkpixcomm.h"
 
 static void gtk_pixcomm_class_init (GtkPixcommClass *cclass);
@@ -52,7 +56,7 @@ gtk_pixcomm_get_type (void)
     {
       static const GtkTypeInfo pixcomm_info =
       {
-	"GtkPixcomm",
+	(char *)"GtkPixcomm",
 	sizeof (GtkPixcomm),
 	sizeof (GtkPixcommClass),
 	(GtkClassInitFunc) gtk_pixcomm_class_init,
@@ -218,7 +222,7 @@ gtk_pixcomm_copyto (GtkPixcomm *pixcomm, SPRITE *src,
   else
   {
     gdk_draw_rectangle (pixcomm->mask, pixcomm->mask_fg_gc, TRUE,
-			0, 0, src->width, src->height);
+			x, y, src->width, src->height);
   }
 
   if (refresh)
