@@ -18,19 +18,20 @@
 #include "mapctrl_g.h"
 
 struct unit;
+struct t_popup_pos {int xroot, yroot;};
 
-gint key_city_workers(GtkWidget *w, GdkEventKey *ev);
-gint adjust_workers(GtkWidget *widget, GdkEventButton *ev);
+void key_city_workers(GtkWidget *w, GdkEventKey *ev);
 
+gint butt_release_mapcanvas(GtkWidget *w, GdkEventButton *ev);
 gint butt_down_mapcanvas(GtkWidget *w, GdkEventButton *ev);
-gint butt_down_wakeup(GtkWidget *w, GdkEventButton *ev);
 gint butt_down_overviewcanvas(GtkWidget *w, GdkEventButton *ev);
 gint move_mapcanvas(GtkWidget *widget, GdkEventButton *event);
-void update_line(int window_x, int window_y);
+gint move_overviewcanvas(GtkWidget *widget, GdkEventButton *event);
 
 void center_on_unit(void);
-void focus_to_next_unit(void);
 void popupinfo_popdown_callback(GtkWidget *w, gpointer data);
+void popupinfo_positioning_callback(GtkWidget *w, GtkAllocation *alloc, 
+                                    gpointer user_data);
 
 /* Color to use to display the workers */
 extern int city_workers_color;
