@@ -10,13 +10,14 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 ***********************************************************************/
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
+#include <assert.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <assert.h>
 
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
@@ -26,7 +27,6 @@
 #include <X11/Xaw/List.h>
 #include <X11/Xaw/SimpleMenu.h>
 #include <X11/Xaw/MenuButton.h>
-#include <X11/Xaw/SimpleMenu.h>
 #include <X11/Xaw/SmeBSB.h>
 #include <X11/Xaw/SmeLine.h>
 #include <X11/Xaw/AsciiText.h>  
@@ -34,7 +34,6 @@
 
 #include "fcintl.h"
 #include "game.h"
-#include "genlist.h"
 #include "government.h"
 #include "map.h"
 #include "mem.h"
@@ -100,15 +99,22 @@ struct Diplomacy_dialog {
 static char *dummy_clause_list_strings[]
     = { "\n", "\n", "\n", "\n", "\n", "\n", 0 };
 
-static struct genlist diplomacy_dialogs;
-static int diplomacy_dialogs_list_has_been_initialised;
+#define SPECLIST_TAG dialog
+#define SPECLIST_TYPE struct Diplomacy_dialog
+#include "speclist.h"
+
+#define dialog_list_iterate(dialoglist, pdialog) \
+    TYPED_LIST_ITERATE(struct Diplomacy_dialog, dialoglist, pdialog)
+#define dialog_list_iterate_end  LIST_ITERATE_END
+
+static struct dialog_list dialog_list;
+static bool dialog_list_list_has_been_initialised = FALSE;
 
 struct Diplomacy_dialog *create_diplomacy_dialog(struct player *plr0, 
 						 struct player *plr1);
 
-struct Diplomacy_dialog *find_diplomacy_dialog(struct player *plr0, 
-					       struct player *plr1);
-void popup_diplomacy_dialog(struct player *plr0, struct player *plr1);
+static struct Diplomacy_dialog *find_diplomacy_dialog(int other_player_id);
+static void popup_diplomacy_dialog(int other_player_id);
 void diplomacy_dialog_close_callback(Widget w, XtPointer client_data, 
 				    XtPointer call_data);
 void diplomacy_dialog_map_callback(Widget w, XtPointer client_data, 
@@ -138,91 +144,95 @@ void update_diplomacy_dialog(struct Diplomacy_dialog *pdialog);
 /****************************************************************
 ...
 *****************************************************************/
-void handle_diplomacy_accept_treaty(struct packet_diplomacy_info *pa)
+void handle_diplomacy_accept_treaty(int counterpart, bool I_accepted,
+				    bool other_accepted)
 {
-  struct Diplomacy_dialog *pdialog;
-  
-  if((pdialog=find_diplomacy_dialog(&game.players[pa->plrno0],
-				&game.players[pa->plrno1]))) {
-    if(pa->plrno_from==game.player_idx)
-      pdialog->treaty.accept0=!pdialog->treaty.accept0;
-    else
-      pdialog->treaty.accept1=!pdialog->treaty.accept1;
-    update_diplomacy_dialog(pdialog);
-  }
-}
+  struct Diplomacy_dialog *pdialog = find_diplomacy_dialog(counterpart);
 
-/****************************************************************
-...
-*****************************************************************/
-void handle_diplomacy_init_meeting(struct packet_diplomacy_info *pa)
-{
-  popup_diplomacy_dialog(&game.players[pa->plrno0], 
-			 &game.players[pa->plrno1]);
-}
-
-/****************************************************************
-...
-*****************************************************************/
-void handle_diplomacy_cancel_meeting(struct packet_diplomacy_info *pa)
-{
-  struct Diplomacy_dialog *pdialog;
-  
-  if((pdialog=find_diplomacy_dialog(&game.players[pa->plrno0],
-				    &game.players[pa->plrno1])))
-    close_diplomacy_dialog(pdialog);
-}
-
-/****************************************************************
-...
-*****************************************************************/
-void handle_diplomacy_create_clause(struct packet_diplomacy_info *pa)
-{
-  struct Diplomacy_dialog *pdialog;
-  
-  if((pdialog=find_diplomacy_dialog(&game.players[pa->plrno0],
-				&game.players[pa->plrno1]))) {
-    add_clause(&pdialog->treaty, &game.players[pa->plrno_from],
-	       pa->clause_type, pa->value);
-    update_diplomacy_dialog(pdialog);
+  if (!pdialog) {
+    return;
   }
 
+  pdialog->treaty.accept0 = I_accepted;
+  pdialog->treaty.accept1 = other_accepted;
+
+  update_diplomacy_dialog(pdialog);
 }
 
 /****************************************************************
 ...
 *****************************************************************/
-void handle_diplomacy_remove_clause(struct packet_diplomacy_info *pa)
+void handle_diplomacy_init_meeting(int counterpart, int initiated_from)
 {
-  struct Diplomacy_dialog *pdialog;
-
-  if((pdialog=find_diplomacy_dialog(&game.players[pa->plrno0],
-				&game.players[pa->plrno1]))) {
-    remove_clause(&pdialog->treaty, &game.players[pa->plrno_from],
-		  pa->clause_type, pa->value);
-    
-    update_diplomacy_dialog(pdialog);
-  }
-
+  popup_diplomacy_dialog(counterpart);
 }
 
+/****************************************************************
+...
+*****************************************************************/
+void handle_diplomacy_cancel_meeting(int counterpart, int initiated_from)
+{
+  struct Diplomacy_dialog *pdialog = find_diplomacy_dialog(counterpart);
 
+  if (!pdialog) {
+    return;
+  }
 
+  close_diplomacy_dialog(pdialog);
+}
+
+/****************************************************************
+...
+*****************************************************************/
+void handle_diplomacy_create_clause(int counterpart, int giver,
+				    enum clause_type type, int value)
+{
+  struct Diplomacy_dialog *pdialog = find_diplomacy_dialog(counterpart);
+
+  if (!pdialog) {
+    return;
+  }
+
+  add_clause(&pdialog->treaty, get_player(giver), type, value);
+  update_diplomacy_dialog(pdialog);
+}
+
+/****************************************************************
+...
+*****************************************************************/
+void handle_diplomacy_remove_clause(int counterpart, int giver,
+				    enum clause_type type, int value)
+{
+  struct Diplomacy_dialog *pdialog = find_diplomacy_dialog(counterpart);
+
+  if (!pdialog) {
+    return;
+  }
+
+  remove_clause(&pdialog->treaty, get_player(giver), type, value);
+  update_diplomacy_dialog(pdialog);
+}
 
 /****************************************************************
 popup the dialog 10% inside the main-window 
 *****************************************************************/
-void popup_diplomacy_dialog(struct player *plr0, struct player *plr1)
+static void popup_diplomacy_dialog(int other_player_id)
 {
-  Position x, y;
-  Dimension width, height;
-  struct Diplomacy_dialog *pdialog;
-  
-  if(!(pdialog=find_diplomacy_dialog(plr0, plr1))) {
-    pdialog=create_diplomacy_dialog(plr0, plr1);
+  struct Diplomacy_dialog *pdialog = find_diplomacy_dialog(other_player_id);
+
+  if (game.player_ptr->ai.control) {
+    return;			/* Don't show if we are AI controlled. */
+  }
+
+  if (!pdialog) {
+    Position x, y;
+    Dimension width, height;
+
+    pdialog = create_diplomacy_dialog(game.player_ptr,
+				      get_player(other_player_id));
     XtVaGetValues(toplevel, XtNwidth, &width, XtNheight, &height, NULL);
-    XtTranslateCoords(toplevel, (Position) width/10, (Position) height/10,
-		      &x, &y);
+    XtTranslateCoords(toplevel, (Position) width / 10,
+		      (Position) height / 10, &x, &y);
     XtVaSetValues(pdialog->dip_dialog_shell, XtNx, x, XtNy, y, NULL);
   }
 
@@ -239,15 +249,17 @@ static int fill_diplomacy_tech_menu(Widget popupmenu,
   int i, flag;
   
   for(i=A_FIRST, flag=0; i<game.num_tech_types; i++) {
-    if(get_invention(plr0, i)==TECH_KNOWN && 
-       (get_invention(plr1, i)==TECH_UNKNOWN || 
-	get_invention(plr1, i)==TECH_REACHABLE)) {
+    if (get_invention(plr0, i) == TECH_KNOWN
+        && (get_invention(plr1, i) == TECH_UNKNOWN
+	    || get_invention(plr1, i) == TECH_REACHABLE)
+        && tech_is_available(plr1, i)) {
       Widget entry=
 	XtVaCreateManagedWidget(advances[i].name, smeBSBObjectClass, 
 				popupmenu, NULL);
       XtAddCallback(entry, XtNcallback, diplomacy_dialog_tech_callback,
-		    INT_TO_XTPOINTER(plr0->player_no * 10000 +
-				     plr1->player_no * 100 + i));
+			 INT_TO_XTPOINTER((plr0->player_no << 24) |
+					 (plr1->player_no << 16) |
+					 i));
       flag=1;
     }
   }
@@ -273,7 +285,7 @@ static int fill_diplomacy_city_menu(Widget popupmenu,
   }
 
   city_list_iterate(plr0->cities, pcity) {
-    if(!city_got_effect(pcity, B_PALACE)){
+    if (!is_capital(pcity)) {
       city_list_ptrs[i] = pcity;
       i++;
     }
@@ -286,9 +298,9 @@ static int fill_diplomacy_city_menu(Widget popupmenu,
       XtVaCreateManagedWidget(city_list_ptrs[j]->name, smeBSBObjectClass, 
 			      popupmenu, NULL);
     XtAddCallback(entry, XtNcallback, diplomacy_dialog_city_callback,
-		  INT_TO_XTPOINTER(city_list_ptrs[j]->id * 1024
-				   + plr0->player_no * 32
-				   + plr1->player_no));
+		  INT_TO_XTPOINTER((plr0->player_no << 24) |
+				   (plr1->player_no << 16) |
+				   city_list_ptrs[j]->id));
   }
   free(city_list_ptrs);
   return i;
@@ -308,7 +320,7 @@ struct Diplomacy_dialog *create_diplomacy_dialog(struct player *plr0,
   Widget entry;
 
   pdialog=fc_malloc(sizeof(struct Diplomacy_dialog));
-  genlist_insert(&diplomacy_dialogs, pdialog, 0);
+  dialog_list_insert(&dialog_list, pdialog);
   
   init_treaty(&pdialog->treaty, plr0, plr1);
   
@@ -671,75 +683,64 @@ void update_diplomacy_dialog(struct Diplomacy_dialog *pdialog)
 /****************************************************************
 ...
 *****************************************************************/
-void diplomacy_dialog_tech_callback(Widget w, XtPointer client_data, 
+void diplomacy_dialog_tech_callback(Widget w, XtPointer client_data,
 				    XtPointer call_data)
 {
-  size_t choice;
-  struct packet_diplomacy_info pa;
-  
-  choice=(size_t)client_data;
+  size_t choice = (size_t) client_data;
+  int giver = (choice >> 24) & 0xff, dest = (choice >> 16) & 0xff, other;
+  int tech = choice & 0xffff;
 
-  pa.plrno0=choice/10000;
-  pa.plrno1=(choice/100)%100;
-  pa.clause_type=CLAUSE_ADVANCE;
-  pa.plrno_from=pa.plrno0;
-  pa.value=choice%100;
-    
-  send_packet_diplomacy_info(&aconnection, PACKET_DIPLOMACY_CREATE_CLAUSE,
-			     &pa);
+  if (giver == game.player_idx) {
+    other = dest;
+  } else {
+    other = giver;
+  }
+
+  dsend_packet_diplomacy_create_clause_req(&aconnection, other, giver,
+					   CLAUSE_ADVANCE, tech);
 }
 
 /****************************************************************
 Callback for trading cities
                               - Kris Bubendorfer
 *****************************************************************/
-void diplomacy_dialog_city_callback(Widget w, XtPointer client_data, 
+void diplomacy_dialog_city_callback(Widget w, XtPointer client_data,
 				    XtPointer call_data)
 {
-  size_t choice;
-  struct packet_diplomacy_info pa;
-  
-  choice=(size_t)client_data;
+  size_t choice = (size_t) client_data;
+  int giver = (choice >> 24) & 0xff, dest = (choice >> 16) & 0xff, other;
+  int city = choice & 0xffff;
 
-  pa.value = choice/1024;
-  choice -= pa.value * 1024;
-  pa.plrno0 = choice/32;
-  choice -= pa.plrno0 * 32;
-  pa.plrno1 = choice;
- 
-  pa.clause_type=CLAUSE_CITY;
-  pa.plrno_from=pa.plrno0;
-    
-  send_packet_diplomacy_info(&aconnection, PACKET_DIPLOMACY_CREATE_CLAUSE,
-			     &pa);
+  if (giver == game.player_idx) {
+    other = dest;
+  } else {
+    other = giver;
+  }
+
+  dsend_packet_diplomacy_create_clause_req(&aconnection, other, giver,
+					   CLAUSE_CITY, city);
 }
 
-
 /****************************************************************
 ...
 *****************************************************************/
 void diplomacy_dialog_erase_clause_callback(Widget w, XtPointer client_data, 
 					    XtPointer call_data)
 {
-  struct Diplomacy_dialog *pdialog=(struct Diplomacy_dialog *)client_data;
-  XawListReturnStruct *ret;
+  struct Diplomacy_dialog *pdialog = (struct Diplomacy_dialog *) client_data;
+  XawListReturnStruct *ret = XawListShowCurrent(pdialog->dip_clauselist);
 
-  ret=XawListShowCurrent(pdialog->dip_clauselist);
-
-  if(ret->list_index!=XAW_LIST_NONE) {
+  if (ret->list_index != XAW_LIST_NONE) {
     int i = 0;
 
     clause_list_iterate(pdialog->treaty.clauses, pclause) {
-      if(i==ret->list_index) {
-	struct packet_diplomacy_info pa;
-
-	pa.plrno0=pdialog->treaty.plr0->player_no;
-	pa.plrno1=pdialog->treaty.plr1->player_no;
-	pa.plrno_from=pclause->from->player_no;
-	pa.clause_type=pclause->type;
-	pa.value=pclause->value;
-	send_packet_diplomacy_info(&aconnection, PACKET_DIPLOMACY_REMOVE_CLAUSE,
-				   &pa);
+      if (i == ret->list_index) {
+	dsend_packet_diplomacy_remove_clause_req(&aconnection,
+						 pdialog->treaty.plr1->
+						 player_no,
+						 pclause->from->player_no,
+						 pclause->type,
+						 pclause->value);
 	return;
       }
       i++;
@@ -747,51 +748,39 @@ void diplomacy_dialog_erase_clause_callback(Widget w, XtPointer client_data,
   }
 }
 
-
-
-
 /****************************************************************
 ...
 *****************************************************************/
-void diplomacy_dialog_map_callback(Widget w, XtPointer client_data, 
+void diplomacy_dialog_map_callback(Widget w, XtPointer client_data,
 				   XtPointer call_data)
 {
-  struct Diplomacy_dialog *pdialog=(struct Diplomacy_dialog *)client_data;
-  struct packet_diplomacy_info pa;
-  struct player *pgiver;
-  
-  pgiver=(XtParent(XtParent(w))==pdialog->dip_map_menubutton0) ? 
-    pdialog->treaty.plr0 : pdialog->treaty.plr1;
-  
-  pa.plrno0=pdialog->treaty.plr0->player_no;
-  pa.plrno1=pdialog->treaty.plr1->player_no;
-  pa.clause_type=CLAUSE_MAP;
-  pa.plrno_from=pgiver->player_no;
-  pa.value=0;
-  send_packet_diplomacy_info(&aconnection, PACKET_DIPLOMACY_CREATE_CLAUSE,
-			     &pa);
+  struct Diplomacy_dialog *pdialog = (struct Diplomacy_dialog *) client_data;
+  struct player *pgiver =
+      (XtParent(XtParent(w)) ==
+       pdialog->dip_map_menubutton0) ? pdialog->treaty.plr0 : pdialog->
+      treaty.plr1;
+
+  dsend_packet_diplomacy_create_clause_req(&aconnection,
+					   pdialog->treaty.plr1->player_no,
+					   pgiver->player_no, CLAUSE_MAP, 0);
 }
 
 /****************************************************************
 ...
 *****************************************************************/
-void diplomacy_dialog_seamap_callback(Widget w, XtPointer client_data, 
-				   XtPointer call_data)
+void diplomacy_dialog_seamap_callback(Widget w, XtPointer client_data,
+				      XtPointer call_data)
 {
-  struct Diplomacy_dialog *pdialog=(struct Diplomacy_dialog *)client_data;
-  struct packet_diplomacy_info pa;
-  struct player *pgiver;
-  
-  pgiver=(XtParent(XtParent(w))==pdialog->dip_map_menubutton0) ? 
-    pdialog->treaty.plr0 : pdialog->treaty.plr1;
-  
-  pa.plrno0=pdialog->treaty.plr0->player_no;
-  pa.plrno1=pdialog->treaty.plr1->player_no;
-  pa.clause_type=CLAUSE_SEAMAP;
-  pa.plrno_from=pgiver->player_no;
-  pa.value=0;
-  send_packet_diplomacy_info(&aconnection, PACKET_DIPLOMACY_CREATE_CLAUSE,
-			     &pa);
+  struct Diplomacy_dialog *pdialog = (struct Diplomacy_dialog *) client_data;
+  struct player *pgiver =
+      (XtParent(XtParent(w)) ==
+       pdialog->dip_map_menubutton0) ? pdialog->treaty.plr0 : pdialog->
+      treaty.plr1;
+
+  dsend_packet_diplomacy_create_clause_req(&aconnection,
+					   pdialog->treaty.plr1->player_no,
+					   pgiver->player_no, CLAUSE_SEAMAP,
+					   0);
 }
 
 /****************************************************************
@@ -800,20 +789,14 @@ void diplomacy_dialog_seamap_callback(Widget w, XtPointer client_data,
 void diplomacy_dialog_vision_callback(Widget w, XtPointer client_data, 
 				   XtPointer call_data)
 {
-  struct Diplomacy_dialog *pdialog=(struct Diplomacy_dialog *)client_data;
-  struct packet_diplomacy_info pa;
-  struct player *pgiver;
-  
-  pgiver = w==pdialog->dip_vision_button0 ? 
-    pdialog->treaty.plr0 : pdialog->treaty.plr1;
-  
-  pa.plrno0=pdialog->treaty.plr0->player_no;
-  pa.plrno1=pdialog->treaty.plr1->player_no;
-  pa.clause_type=CLAUSE_VISION;
-  pa.plrno_from=pgiver->player_no;
-  pa.value=0;
-  send_packet_diplomacy_info(&aconnection, PACKET_DIPLOMACY_CREATE_CLAUSE,
-			     &pa);
+  struct Diplomacy_dialog *pdialog = (struct Diplomacy_dialog *) client_data;
+  struct player *pgiver = (w == pdialog->dip_vision_button0) ?
+      pdialog->treaty.plr0 : pdialog->treaty.plr1;
+
+  dsend_packet_diplomacy_create_clause_req(&aconnection,
+					   pdialog->treaty.plr1->player_no,
+					   pgiver->player_no, CLAUSE_VISION,
+					   0);
 }
 
 /****************************************************************
@@ -822,20 +805,13 @@ Generic add-a-clause function for adding pact types
 static void diplomacy_dialog_add_pact_clause(Widget w, XtPointer client_data,
 					     XtPointer call_data, int type)
 {
-  struct Diplomacy_dialog *pdialog=(struct Diplomacy_dialog *)client_data;
-  struct packet_diplomacy_info pa;
-  struct player *pgiver;
-  
-  pgiver=(XtParent(XtParent(w))==pdialog->dip_pact_menubutton) ? 
-    pdialog->treaty.plr0 : pdialog->treaty.plr1;
-  
-  pa.plrno0=pdialog->treaty.plr0->player_no;
-  pa.plrno1=pdialog->treaty.plr1->player_no;
-  pa.clause_type=type;
-  pa.plrno_from=pgiver->player_no;
-  pa.value=0;
-  send_packet_diplomacy_info(&aconnection, PACKET_DIPLOMACY_CREATE_CLAUSE,
-			     &pa);  
+  struct Diplomacy_dialog *pdialog = (struct Diplomacy_dialog *) client_data;
+  struct player *pgiver = (w == pdialog->dip_vision_button0) ?
+      pdialog->treaty.plr0 : pdialog->treaty.plr1;
+
+  dsend_packet_diplomacy_create_clause_req(&aconnection,
+					   pdialog->treaty.plr1->player_no,
+					   pgiver->player_no, type, 0);
 }
 
 /****************************************************************
@@ -875,18 +851,13 @@ void diplomacy_dialog_alliance_callback(Widget w, XtPointer client_data,
 /****************************************************************
 ...
 *****************************************************************/
-void diplomacy_dialog_close_callback(Widget w, XtPointer client_data, 
+void diplomacy_dialog_close_callback(Widget w, XtPointer client_data,
 				     XtPointer call_data)
 {
-  struct Diplomacy_dialog *pdialog=(struct Diplomacy_dialog *)client_data;
-  struct packet_diplomacy_info pa;
+  struct Diplomacy_dialog *pdialog = (struct Diplomacy_dialog *) client_data;
 
-  pa.plrno0=game.player_idx;
-  pa.plrno1=pdialog->treaty.plr1->player_no;
-  pa.plrno_from=pa.plrno0;
-  send_packet_diplomacy_info(&aconnection, PACKET_DIPLOMACY_CANCEL_MEETING, 
-			     &pa);
-  
+  dsend_packet_diplomacy_cancel_meeting_req(&aconnection,
+					    pdialog->treaty.plr1->player_no);
   close_diplomacy_dialog(pdialog);
 }
 
@@ -897,14 +868,10 @@ void diplomacy_dialog_close_callback(Widget w, XtPointer client_data,
 void diplomacy_dialog_accept_callback(Widget w, XtPointer client_data, 
 				      XtPointer call_data)
 {
-  struct Diplomacy_dialog *pdialog=(struct Diplomacy_dialog *)client_data;
-  struct packet_diplomacy_info pa;
-  
-  pa.plrno0=pdialog->treaty.plr0->player_no;
-  pa.plrno1=pdialog->treaty.plr1->player_no;
-  pa.plrno_from=game.player_idx;
-  send_packet_diplomacy_info(&aconnection, PACKET_DIPLOMACY_ACCEPT_TREATY,
-			     &pa);
+  struct Diplomacy_dialog *pdialog = (struct Diplomacy_dialog *) client_data;
+
+  dsend_packet_diplomacy_accept_treaty_req(&aconnection,
+					   pdialog->treaty.plr1->player_no);
 }
 
 
@@ -915,33 +882,30 @@ void close_diplomacy_dialog(struct Diplomacy_dialog *pdialog)
 {
   XtDestroyWidget(pdialog->dip_dialog_shell);
   
-  genlist_unlink(&diplomacy_dialogs, pdialog);
+  dialog_list_unlink(&dialog_list, pdialog);
   free(pdialog);
 }
 
 /*****************************************************************
 ...
 *****************************************************************/
-struct Diplomacy_dialog *find_diplomacy_dialog(struct player *plr0, 
-					       struct player *plr1)
+static struct Diplomacy_dialog *find_diplomacy_dialog(int other_player_id)
 {
-  struct genlist_iterator myiter;
+  struct player *plr0 = game.player_ptr, *plr1 = get_player(other_player_id);
 
-  if(!diplomacy_dialogs_list_has_been_initialised) {
-    genlist_init(&diplomacy_dialogs);
-    diplomacy_dialogs_list_has_been_initialised=1;
+  if (!dialog_list_list_has_been_initialised) {
+    dialog_list_init(&dialog_list);
+    dialog_list_list_has_been_initialised = TRUE;
   }
-  
-  genlist_iterator_init(&myiter, &diplomacy_dialogs, 0);
-    
-  for(; ITERATOR_PTR(myiter); ITERATOR_NEXT(myiter)) {
-    struct Diplomacy_dialog *pdialog=
-      (struct Diplomacy_dialog *)ITERATOR_PTR(myiter);
-    if((pdialog->treaty.plr0==plr0 && pdialog->treaty.plr1==plr1) ||
-       (pdialog->treaty.plr0==plr1 && pdialog->treaty.plr1==plr0))
+
+  dialog_list_iterate(dialog_list, pdialog) {
+    if ((pdialog->treaty.plr0 == plr0 && pdialog->treaty.plr1 == plr1) ||
+	(pdialog->treaty.plr0 == plr1 && pdialog->treaty.plr1 == plr0)) {
       return pdialog;
-  }
-  return 0;
+    }
+  } dialog_list_iterate_end;
+
+  return NULL;
 }
 
 /*****************************************************************
@@ -949,18 +913,13 @@ struct Diplomacy_dialog *find_diplomacy_dialog(struct player *plr0,
 *****************************************************************/
 static struct Diplomacy_dialog *find_diplomacy_by_input(Widget w)
 {
-  struct genlist_iterator myiter;
-  
-  genlist_iterator_init(&myiter, &diplomacy_dialogs, 0);
-    
-  for(; ITERATOR_PTR(myiter); ITERATOR_NEXT(myiter)) {
-    struct Diplomacy_dialog *pdialog=
-      (struct Diplomacy_dialog *)ITERATOR_PTR(myiter);
-    if((pdialog->dip_gold_input0==w) || (pdialog->dip_gold_input1==w)) {
+  dialog_list_iterate(dialog_list, pdialog) {
+    if ((pdialog->dip_gold_input0 == w) || (pdialog->dip_gold_input1 == w)) {
       return pdialog;
     }
-  }
-  return 0;
+  } dialog_list_iterate_end;
+
+  return NULL;
 }
 
 /*****************************************************************
@@ -968,27 +927,21 @@ static struct Diplomacy_dialog *find_diplomacy_by_input(Widget w)
 *****************************************************************/
 void diplodlg_key_gold(Widget w)
 {
-  struct Diplomacy_dialog *pdialog;
+  struct Diplomacy_dialog *pdialog = find_diplomacy_by_input(w);
   
-  if((pdialog=find_diplomacy_by_input(w))) {
-    struct player *pgiver;
+  if (pdialog) {
+    struct player *pgiver = (w == pdialog->dip_gold_input0) ?
+	pdialog->treaty.plr0 : pdialog->treaty.plr1;
     XtPointer dp;
     int amount;
     
-    pgiver=(w==pdialog->dip_gold_input0) ? 
-      pdialog->treaty.plr0 : pdialog->treaty.plr1;
-    
     XtVaGetValues(w, XtNstring, &dp, NULL);
     if (sscanf(dp, "%d", &amount) == 1 && amount >= 0
 	&& amount <= pgiver->economic.gold) {
-      struct packet_diplomacy_info pa;
-      pa.plrno0=pdialog->treaty.plr0->player_no;
-      pa.plrno1=pdialog->treaty.plr1->player_no;
-      pa.clause_type=CLAUSE_GOLD;
-      pa.plrno_from=pgiver->player_no;
-      pa.value=amount;
-      send_packet_diplomacy_info(&aconnection, PACKET_DIPLOMACY_CREATE_CLAUSE,
-				 &pa);
+      dsend_packet_diplomacy_create_clause_req(&aconnection,
+					       pdialog->treaty.plr1->
+					       player_no, pgiver->player_no,
+					       CLAUSE_GOLD, amount);
       XtVaSetValues(w, XtNstring, "", NULL);
     }
     else
@@ -1001,13 +954,11 @@ void diplodlg_key_gold(Widget w)
 *****************************************************************/
 void close_all_diplomacy_dialogs(void)
 {
-  struct Diplomacy_dialog *pdialog;
-  
-  if (!diplomacy_dialogs_list_has_been_initialised) {
+  if (!dialog_list_list_has_been_initialised) {
     return;
   }
-  while (genlist_size(&diplomacy_dialogs)) {
-    pdialog = genlist_get(&diplomacy_dialogs, 0);
-    close_diplomacy_dialog(pdialog);
+
+  while (dialog_list_size(&dialog_list) > 0) {
+    close_diplomacy_dialog(dialog_list_get(&dialog_list, 0));
   }
 }
