@@ -254,6 +254,7 @@ void handle_unit_remove(int unit_id)
 void handle_nuke_tile_info(int x, int y)
 {
   flush_dirty();
+  redraw_selection_rectangle();
   put_nuke_mushroom_pixmaps(map_pos_to_tile(x, y));
 }
 
@@ -292,6 +293,7 @@ void handle_unit_combat_info(int attacker_unit_id, int defender_unit_id,
 
       if (do_combat_animation) {
 	flush_dirty();
+	redraw_selection_rectangle();
 	decrease_unit_hp_smooth(punit0, hp0, punit1, hp1);
 	if (make_winner_veteran) {
 	  pwinner->veteran++;
@@ -640,8 +642,9 @@ static void handle_city_packet_common(struct city *pcity, bool is_new,
     city_workers_display=NULL;
   }
 
-  if (popup &&
-      (!game.player_ptr->ai.control || ai_popup_windows)) {
+  if (popup
+      && (!game.player_ptr->ai.control || ai_popup_windows)
+      && can_client_issue_orders()) {
     update_menus();
     if (!city_dialog_is_open(pcity)) {
       popup_city_dialog(pcity, FALSE);
@@ -981,13 +984,14 @@ static bool handle_unit_packet_common(struct unit *packet_unit)
 {
   struct city *pcity;
   struct unit *punit;
+  bool need_update_menus = FALSE;
   bool repaint_unit = FALSE;
   bool repaint_city = FALSE;	/* regards unit's homecity */
   struct tile *old_tile = NULL;
   bool check_focus = FALSE;     /* conservative focus change */
   bool moved = FALSE;
   bool ret = FALSE;
-  int old_transported_by;
+  struct unit *focus_unit = get_unit_in_focus();
   
   punit = player_find_unit_by_id(get_player(packet_unit->owner),
 				 packet_unit->id);
@@ -995,7 +999,6 @@ static bool handle_unit_packet_common(struct unit *packet_unit)
   if (punit) {
     ret = TRUE;
     punit->activity_count = packet_unit->activity_count;
-    punit->occupy = packet_unit->occupy;
     if (punit->ai.control != packet_unit->ai.control) {
       punit->ai.control = packet_unit->ai.control;
       repaint_unit = TRUE;
@@ -1008,6 +1011,8 @@ static bool handle_unit_packet_common(struct unit *packet_unit)
 
     if (punit->activity != packet_unit->activity
 	|| punit->activity_target != packet_unit->activity_target
+	|| punit->transported_by != packet_unit->transported_by
+	|| punit->occupy != packet_unit->occupy
 	|| punit->has_orders != packet_unit->has_orders
 	|| punit->orders.repeat != packet_unit->orders.repeat
 	|| punit->orders.vigilant != packet_unit->orders.vigilant
@@ -1048,6 +1053,16 @@ static bool handle_unit_packet_common(struct unit *packet_unit)
       punit->activity = packet_unit->activity;
       punit->activity_target = packet_unit->activity_target;
 
+      if (punit->occupy != packet_unit->occupy
+	  && focus_unit && focus_unit->tile == packet_unit->tile) {
+	/* Special case: (un)loading a unit in a transporter on the
+	 * same tile as the focus unit may (dis)allow the focus unit to be
+	 * loaded.  Thus the orders->(un)load menu item needs updating. */
+	need_update_menus = TRUE;
+      }
+      punit->occupy = packet_unit->occupy;
+      punit->transported_by = packet_unit->transported_by;
+
       punit->has_orders = packet_unit->has_orders;
       punit->orders.length = packet_unit->orders.length;
       punit->orders.index = packet_unit->orders.index;
@@ -1066,26 +1081,10 @@ static bool handle_unit_packet_common(struct unit *packet_unit)
       }
     } /*** End of Change in activity or activity's target. ***/
 
-    old_transported_by = punit->transported_by;
-    punit->transported_by = packet_unit->transported_by;
-    if (old_transported_by != packet_unit->transported_by) {
-      struct unit *ptrans;
-
-      if (old_transported_by != -1
-	  && (ptrans = find_unit_by_id(old_transported_by))) {
-	refresh_unit_city_dialogs(ptrans);
-      }
-
-      if (packet_unit->transported_by != -1
-          && (ptrans = find_unit_by_id(packet_unit->transported_by))) {
-	refresh_unit_city_dialogs(ptrans);
-      }
-    }
-    
     /* These two lines force the menus to be updated as appropriate when
      * the focus unit changes. */
     if (punit == get_unit_in_focus()) {
-      update_menus();
+      need_update_menus = TRUE;
     }
 
     if (punit->homecity != packet_unit->homecity) {
@@ -1121,7 +1120,7 @@ static bool handle_unit_packet_common(struct unit *packet_unit)
       }
       if(punit == get_unit_in_focus()) {
         /* Update the orders menu -- the unit might have new abilities */
-        update_menus();
+	need_update_menus = TRUE;
       }
     }
 
@@ -1183,6 +1182,7 @@ static bool handle_unit_packet_common(struct unit *packet_unit)
 	   && (!game.player_ptr->ai.control || ai_popup_windows)
 	   && punit->owner==game.player_idx
 	   && !unit_has_orders(punit)
+	   && can_client_issue_orders()
 	   && (unit_can_help_build_wonder_here(punit)
 	       || unit_can_est_traderoute_here(punit))) {
 	  process_caravan_arrival(punit);
@@ -1289,6 +1289,10 @@ static bool handle_unit_packet_common(struct unit *packet_unit)
     update_unit_focus();
   }
 
+  if (need_update_menus) {
+    update_menus();
+  }
+
   return ret;
 }
 
@@ -1475,7 +1479,7 @@ void handle_game_info(struct packet_game_info *pinfo)
 ...
 **************************************************************************/
 static bool read_player_info_techs(struct player *pplayer,
-				   unsigned char *inventions)
+				   char *inventions)
 {
   bool need_effect_update = FALSE;
 
@@ -2518,7 +2522,7 @@ void handle_ruleset_terrain(struct packet_ruleset_terrain *p)
 	    p->id);
     return;
   }
-  t = &(tile_types[p->id]);
+  t = get_tile_type(p->id);
 
   sz_strlcpy(t->terrain_name_orig, p->terrain_name);
   t->terrain_name = t->terrain_name_orig;
