@@ -18,10 +18,10 @@
    id values to unit and city pointers.
 ***************************************************************************/
 
-struct city;
-struct unit;
+#include "fc_types.h"
 
 void idex_init(void);
+void idex_free(void);
 
 void idex_register_city(struct city *pcity);
 void idex_register_unit(struct unit *punit);
