@@ -136,7 +136,6 @@ struct tile_type {
 
   char *helptext;
 };
-extern struct tile_type tile_types[MAX_NUM_TERRAINS];
 
 /* The direction8 gives the 8 possible directions.  These may be used in
  * a number of ways, for instance as an index into the DIR_DX/DIR_DY
@@ -219,7 +218,6 @@ void map_allocate(void);
 void map_free(void);
 
 const char *map_get_tile_info_text(const struct tile *ptile);
-const char *map_get_tile_fpt_text(const struct tile *ptile);
 
 int map_vector_to_real_distance(int dx, int dy);
 int map_vector_to_sq_distance(int dx, int dy);
