@@ -159,9 +159,9 @@ int send_packet_data(struct connection *pc, unsigned char *data, int len)
 	PACKET_PROCESSING_FINISHED || packet_type == PACKET_THAW_HINT) {
       pc->compression.frozen_level--;
       if (pc->compression.frozen_level == 0) {
-	long int compressed_size = 12 + pc->compression.queue.size * 1.001;
+	uLongf compressed_size = 12 + pc->compression.queue.size * 1.001;
 	int error;
-	char compressed[compressed_size];
+	Bytef compressed[compressed_size];
 
 	error =
 	    compress2(compressed, &compressed_size,
@@ -362,18 +362,18 @@ void *get_packet_from_connection(struct connection *pc,
   }
 #endif
 
-  if (whole_packet_len > pc->buffer->ndata) {
+  if ((unsigned)whole_packet_len > pc->buffer->ndata) {
     return NULL;		/* not all data has been read */
   }
 
 #ifdef USE_COMPRESSION
   if (compressed_packet) {
-    int compressed_size = whole_packet_len - header_size;
+    uLong compressed_size = whole_packet_len - header_size;
     /* 
      * We don't know the decompressed size. We assume a bad case
      * here: an expansion by an factor of 100. 
      */
-    long int decompressed_size = 100 * compressed_size;
+    uLongf decompressed_size = 100 * compressed_size;
     void *decompressed = fc_malloc(decompressed_size);
     int error;
     struct socket_packet_buffer *buffer = pc->buffer;
@@ -420,7 +420,7 @@ void *get_packet_from_connection(struct connection *pc,
 
     buffer->ndata += decompressed_size;
     
-    freelog(COMPRESS_LOG_LEVEL, "COMPRESS: decompressed %d into %ld",
+    freelog(COMPRESS_LOG_LEVEL, "COMPRESS: decompressed %ld into %ld",
 	    compressed_size, decompressed_size);
 
     return get_packet_from_connection(pc, ptype, presult);
@@ -573,17 +573,47 @@ void generic_handle_player_attribute_chunk(struct player *pplayer,
 					   packet_player_attribute_chunk
 					   *chunk)
 {
+  freelog(LOG_DEBUG, "received attribute chunk %d/%d %d", chunk->offset,
+	  chunk->total_length, chunk->chunk_length);
+
+  if (chunk->total_length < 0
+      || chunk->total_length >= MAX_ATTRIBUTE_BLOCK
+      || chunk->offset < 0
+      || chunk->offset + chunk->chunk_length > chunk->total_length
+      || (chunk->offset != 0
+          && chunk->total_length != pplayer->attribute_block_buffer.length)) {
+    /* wrong attribute data */
+    if (pplayer->attribute_block_buffer.data) {
+      free(pplayer->attribute_block_buffer.data);
+      pplayer->attribute_block_buffer.data = NULL;
+    }
+    pplayer->attribute_block_buffer.length = 0;
+    freelog(LOG_ERROR, "Received wrong attribute chunk");
+    return;
+  }
   /* first one in a row */
   if (chunk->offset == 0) {
-    if (pplayer->attribute_block.data) {
-      free(pplayer->attribute_block.data);
-      pplayer->attribute_block.data = NULL;
+    if (pplayer->attribute_block_buffer.data) {
+      free(pplayer->attribute_block_buffer.data);
+      pplayer->attribute_block_buffer.data = NULL;
     }
-    pplayer->attribute_block.data = fc_malloc(chunk->total_length);
-    pplayer->attribute_block.length = chunk->total_length;
+    pplayer->attribute_block_buffer.data = fc_malloc(chunk->total_length);
+    pplayer->attribute_block_buffer.length = chunk->total_length;
   }
-  memcpy((char *) (pplayer->attribute_block.data) + chunk->offset,
+  memcpy((char *) (pplayer->attribute_block_buffer.data) + chunk->offset,
 	 chunk->data, chunk->chunk_length);
+  
+  if (chunk->offset + chunk->chunk_length == chunk->total_length) {
+    /* Received full attribute block */
+    if (pplayer->attribute_block.data != NULL) {
+      free(pplayer->attribute_block.data);
+    }
+    pplayer->attribute_block.data = pplayer->attribute_block_buffer.data;
+    pplayer->attribute_block.length = pplayer->attribute_block_buffer.length;
+    
+    pplayer->attribute_block_buffer.data = NULL;
+    pplayer->attribute_block_buffer.length = 0;
+  }
 }
 
 /**************************************************************************
@@ -682,33 +712,6 @@ void pre_send_packet_player_attribute_chunk(struct connection *pc,
 
 }
 
-void post_receive_packet_player_attribute_chunk(struct connection *pc,
-						struct packet_player_attribute_chunk
-						*packet)
-{
-  /*
-   * Because of the changes in enum packet_type during the 1.12.1
-   * timeframe an old server will trigger the following condition.
-   */
-  if (packet->total_length <= 0
-      || packet->total_length >= MAX_ATTRIBUTE_BLOCK) {
-    freelog(LOG_FATAL, _("The server you tried to connect is too old "
-			 "(1.12.0 or earlier). Please choose another "
-			 "server next time. Good bye."));
-    exit(EXIT_FAILURE);
-  }
-  assert(packet->total_length > 0
-	 && packet->total_length < MAX_ATTRIBUTE_BLOCK);
-  /* 500 bytes header, just to be sure */
-  assert(packet->chunk_length > 0
-	 && packet->chunk_length < MAX_LEN_PACKET - 500);
-  assert(packet->chunk_length <= packet->total_length);
-  assert(packet->offset >= 0 && packet->offset < packet->total_length);
-
-  freelog(LOG_DEBUG, "received attribute chunk %d/%d %d", packet->offset,
-	  packet->total_length, packet->chunk_length);
-}
-
 void post_receive_packet_game_state(struct connection *pc,
 				    struct packet_game_state *packet)
 {
