@@ -256,7 +256,7 @@ reason for rejection. There's 4 different modes:
 bool is_allowed_city_name(struct player *pplayer, const char *city_name,
 			  char *error_buf, size_t bufsz)
 {
-  struct connection *pconn;
+  struct connection *pconn = find_conn_by_user(pplayer->username);
 
   /* Mode 1: A city name has to be unique for each player. */
   if (game.allowed_city_names == 1 &&
@@ -316,8 +316,7 @@ bool is_allowed_city_name(struct player *pplayer, const char *city_name,
    * original nation are exhausted and the backup nations have non-ascii
    * names in them. */
   if (!is_ascii_name(city_name)
-      && (pconn = find_conn_by_user(pplayer->username))
-      && pconn->access_level != ALLOW_HACK) {
+      && (!pconn || pconn->access_level != ALLOW_HACK)) {
     if (error_buf) {
       my_snprintf(error_buf, bufsz,
 		  _("%s is not a valid name. Only ASCII or "
