@@ -842,27 +842,27 @@ static void make_rivers(void)
 	   it. */
 	&& (count_terrain_near_tile(ptile, TRUE, TRUE, T_HILLS)
 	    + count_terrain_near_tile(ptile, TRUE, TRUE, T_MOUNTAINS) < 90
-	    || iteration_counter == RIVERS_MAXTRIES / 10 * 5)
+	    || iteration_counter >= RIVERS_MAXTRIES / 10 * 5)
 
 	/* Don't start a river on hills unless it is hard to find
 	   somewhere else to start it. */
 	&& (map_get_terrain(ptile) != T_HILLS
-	    || iteration_counter == RIVERS_MAXTRIES / 10 * 6)
+	    || iteration_counter >= RIVERS_MAXTRIES / 10 * 6)
 
 	/* Don't start a river on mountains unless it is hard to find
 	   somewhere else to start it. */
 	&& (map_get_terrain(ptile) != T_MOUNTAINS
-	    || iteration_counter == RIVERS_MAXTRIES / 10 * 7)
+	    || iteration_counter >= RIVERS_MAXTRIES / 10 * 7)
 
 	/* Don't start a river on arctic unless it is hard to find
 	   somewhere else to start it. */
 	&& (map_get_terrain(ptile) != T_ARCTIC
-	    || iteration_counter == RIVERS_MAXTRIES / 10 * 8)
+	    || iteration_counter >= RIVERS_MAXTRIES / 10 * 8)
 
 	/* Don't start a river on desert unless it is hard to find
 	   somewhere else to start it. */
 	&& (map_get_terrain(ptile) != T_DESERT
-	    || iteration_counter == RIVERS_MAXTRIES / 10 * 9)) {
+	    || iteration_counter >= RIVERS_MAXTRIES / 10 * 9)) {
 
       /* Reset river_map before making a new river. */
       for (i = 0; i < map.xsize * map.ysize; i++) {
@@ -1102,26 +1102,53 @@ void map_fractal_generate(bool autosize)
   /* We don't want random start positions in a scenario which already
    * provides them. */
   if (map.num_start_positions == 0) {
+    enum start_mode mode = MT_ALL;
+    bool success;
+    
     switch (map.generator) {
     case 0:
     case 1:
-      create_start_positions(map.startpos);
+      mode = map.startpos;
       break;
     case 2:
       if (map.startpos == 0) {
-	create_start_positions(MT_ALL);
+        mode = MT_ALL;
       } else {
-	create_start_positions(map.startpos);
+        mode = map.startpos;
       }
       break;
     case 3:
       if (map.startpos <= 1 || (map.startpos == 4)) {
-	create_start_positions(MT_SINGLE);
+        mode = MT_SINGLE;
       } else {
-	create_start_positions(MT_2or3);
+	mode = MT_2or3;
       }
       break;
     }
+    
+    for(;;) {
+      success = create_start_positions(mode);
+      if (success) {
+        break;
+      }
+      
+      switch(mode) {
+        case MT_SINGLE:
+	  mode = MT_2or3;
+	  break;
+	case MT_2or3:
+	  mode = MT_ALL;
+	  break;
+	case MT_ALL:
+	  mode = MT_VARIABLE;
+	  break;
+	default:
+	  assert(0);
+	  die("The server couldn't allocate starting positions.");
+      }
+    }
+
+
   }
 
   assign_continent_numbers(FALSE);
@@ -1228,20 +1255,20 @@ static void add_specials(int prob)
     if (!is_ocean(ttype)
 	&& !is_special_close(ptile) 
 	&& myrand(1000) < prob) {
-      if (tile_types[ttype].special_1_name[0] != '\0'
-	  && (tile_types[ttype].special_2_name[0] == '\0'
+      if (get_tile_type(ttype)->special_1_name[0] != '\0'
+	  && (get_tile_type(ttype)->special_2_name[0] == '\0'
 	      || (myrand(100) < 50))) {
 	map_set_special(ptile, S_SPECIAL_1);
-      } else if (tile_types[ttype].special_2_name[0] != '\0') {
+      } else if (get_tile_type(ttype)->special_2_name[0] != '\0') {
 	map_set_special(ptile, S_SPECIAL_2);
       }
     } else if (is_ocean(ttype) && near_safe_tiles(ptile) 
 	       && myrand(1000) < prob && !is_special_close(ptile)) {
-      if (tile_types[ttype].special_1_name[0] != '\0'
-	  && (tile_types[ttype].special_2_name[0] == '\0'
+      if (get_tile_type(ttype)->special_1_name[0] != '\0'
+	  && (get_tile_type(ttype)->special_2_name[0] == '\0'
 	      || (myrand(100) < 50))) {
         map_set_special(ptile, S_SPECIAL_1);
-      } else if (tile_types[ttype].special_2_name[0] != '\0') {
+      } else if (get_tile_type(ttype)->special_2_name[0] != '\0') {
 	map_set_special(ptile, S_SPECIAL_2);
       }
     }
@@ -1914,7 +1941,7 @@ static void mapgenerator4(void)
   /* no islands with mass >> sqr(min(xsize,ysize)) */
 
   if (game.nplayers < 2 || map.landpercent > 80) {
-    map.generator = 3;
+    map.startpos = 1;
     return;
   }
 
