@@ -13,6 +13,26 @@
 #ifndef FC__SANITYCHECK_H
 #define FC__SANITYCHECK_H
 
+#include "fc_types.h"
+
+#if ((IS_BETA_VERSION || IS_DEVEL_VERSION) && !defined NDEBUG) \
+  || defined DEBUG
+#  define SANITY_CHECKING
+#endif
+
+#ifdef SANITY_CHECKING
+
+#  define sanity_check_city(x) real_sanity_check_city(x, __FILE__, __LINE__)
+void real_sanity_check_city(struct city *pcity, const char *file, int line);
+
 void sanity_check(void);
 
+#else /* SANITY_CHECKING */
+
+#  define sanity_check_city(x) (void)0
+#  define sanity_check() (void)0
+
+#endif /* SANITY_CHECKING */
+
+
 #endif  /* FC__SANITYCHECK_H */
