@@ -74,6 +74,7 @@
 #include "support.h"
 #include "timing.h"
 
+#include "auth.h"
 #include "connecthand.h"
 #include "console.h"
 #include "meta.h"
@@ -401,6 +402,9 @@ int sniff_packets(void)
       if(conn_list_size(&game.est_connections) == 0) {
 	if (last_noplayers != 0) {
 	  if (time(NULL)>last_noplayers + srvarg.quitidle) {
+	    if (srvarg.exit_on_end) {
+	      save_game_auto();
+	    }
 	    set_meta_message_string("restarting for lack of players");
 	    freelog(LOG_NORMAL, get_meta_message_string());
 	    (void) send_server_info_to_metaserver(META_INFO);
@@ -411,6 +415,11 @@ int sniff_packets(void)
               lost_connection_to_client(pconn);
               close_connection(pconn);
             } conn_list_iterate_end;
+
+	    if (srvarg.exit_on_end) {
+	      /* No need for anything more; just quit. */
+	      server_quit();
+	    }
 	  }
 	} else {
           char buf[256];
@@ -457,10 +466,8 @@ int sniff_packets(void)
 
     /* if we've waited long enough after a failure, respond to the client */
     conn_list_iterate(game.all_connections, pconn) {
-      if (pconn->server.status == AS_FAILED
-          && pconn->server.authentication_stop > 0
-          && time(NULL) >= pconn->server.authentication_stop) {
-        unfail_authentication(pconn);
+      if (srvarg.auth_enabled && pconn->server.status != AS_ESTABLISHED) {
+        process_authentication_status(pconn);
       }
     } conn_list_iterate_end
 
@@ -577,12 +584,10 @@ int sniff_packets(void)
       char buf[BUF_SIZE+1];
       char *buf_internal;
       
-      if((didget=read(0, buf, BUF_SIZE))==-1) {
-	die("read from stdin failed");
-      }
-
-      if(didget==0) {
+      didget = read(0, buf, BUF_SIZE);
+      if (didget <= 0) {
         handle_stdin_close();
+	didget = 0; /* Avoid buffer underrun below. */
       }
 
       *(buf+didget)='\0';
@@ -720,12 +725,13 @@ static const char *makeup_connection_name(int *id)
 ********************************************************************/
 static int server_accept_connection(int sockfd)
 {
-
-# if defined(__VMS) && !defined(_DECC_V4_SOURCE)
-    size_t fromlen;
-# else
-    int fromlen;
-# endif
+  /* This used to have size_t for some platforms.  If this is necessary
+   * it should be done with a configure check not a platform check. */
+#ifdef HAVE_SOCKLEN_T
+  socklen_t fromlen;
+#else
+  int fromlen;
+#endif
 
   int new_sock;
   union my_sockaddr fromend;
@@ -758,8 +764,8 @@ static int server_accept_connection(int sockfd)
       pconn->notify_of_writable_data = NULL;
       pconn->server.currently_processed_request_id = 0;
       pconn->server.last_request_id_seen = 0;
-      pconn->server.authentication_tries = 0;
-      pconn->server.authentication_stop = 0;
+      pconn->server.auth_tries = 0;
+      pconn->server.auth_settime = 0;
       pconn->server.status = AS_NOT_ESTABLISHED;
       pconn->server.ping_timers =
 	  fc_malloc(sizeof(*pconn->server.ping_timers));
@@ -1038,11 +1044,11 @@ static void get_lanserver_announcement(void)
 static void send_lanserver_response(void)
 {
   unsigned char buffer[MAX_LEN_PACKET];
-  unsigned char hostname[512];
-  unsigned char port[256];
-  unsigned char version[256];
-  unsigned char players[256];
-  unsigned char status[256];
+  char hostname[512];
+  char port[256];
+  char version[256];
+  char players[256];
+  char status[256];
   struct data_out dout;
   union my_sockaddr addr;
   int socksend, setting = 1;
