@@ -354,7 +354,7 @@ struct settings_s settings[] = {
 	  GAME_MIN_SEED, GAME_MAX_SEED, GAME_DEFAULT_SEED)
 
   GEN_INT("specials", map.riches,
-	  SSET_MAP_ADD, SSET_ECOLOGY, SSET_VITAL, SSET_TO_CLIENT,
+	  SSET_MAP_ADD, SSET_GEOLOGY, SSET_VITAL, SSET_TO_CLIENT,
 	  N_("Amount of \"special\" resource squares"), 
 	  N_("Special resources improve the basic terrain type they "
 	     "are on. The server variable's scale is parts per "
@@ -861,7 +861,7 @@ struct settings_s settings[] = {
 	   autotoggle_callback, GAME_DEFAULT_AUTO_AI_TOGGLE)
 
   GEN_INT("endyear", game.end_year,
-	  SSET_META, SSET_SOCIOLOGY, SSET_TO_CLIENT, SSET_VITAL,
+	  SSET_META, SSET_SOCIOLOGY, SSET_VITAL, SSET_TO_CLIENT,
 	  N_("Year the game ends"), 
 	  N_("The game will end at the end of the given year."), NULL,
 	  GAME_MIN_END_YEAR, GAME_MAX_END_YEAR, GAME_DEFAULT_END_YEAR)
@@ -877,6 +877,13 @@ struct settings_s settings[] = {
 	     "\"timeoutincrease\" to have a dynamic timer."), NULL, 
 	   GAME_MIN_TIMEOUT, GAME_MAX_TIMEOUT, GAME_DEFAULT_TIMEOUT)
 
+  GEN_INT("timeaddenemymove", game.timeoutaddenemymove,
+        SSET_META, SSET_INTERNAL, SSET_VITAL, SSET_TO_CLIENT,
+	  N_("Timeout at least n seconds when enemy moved"),
+	  N_("Any time a unit moves when in sight of an enemy player, "
+	     "the remaining timeout is set to this value if it was lower."),
+	  NULL, 0, GAME_MAX_TIMEOUT, GAME_DEFAULT_TIMEOUTADDEMOVE)
+  
   GEN_INT("nettimeout", game.tcptimeout,
 	  SSET_META, SSET_NETWORK, SSET_RARE, SSET_TO_CLIENT,
 	  N_("Seconds to let a client's network connection block"),
